@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/compare_db.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/compare_db.h,v 1.2 2006/10/27 18:54:29 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,6 +48,6 @@ void init_rxlst(list* rxlst);
  * compare_dbline()
  * Return RETOK if same RETFAIL if not
  */
-int compare_dbline(db_line* old,db_line* new,int ignorelist);
+int compare_dbline(db_line* old,db_line* new,DB_ATTR_TYPE ignorelist);
 
 #endif
