@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide2/include/conf_lex.h,v 1.3 2002/05/29 08:04:26 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/conf_lex.h,v 1.2 2006/10/27 20:45:57 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,9 +25,9 @@ void conf_put_token(const char* s);
 
 extern int conferror(const char*);
 
-extern int conflex();
+extern int conflex(void);
 
-extern int confparse();
+extern int confparse(void);
 
 extern void* conf_scan_string(char*);
 
