@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/db.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/db.h,v 1.3 2005/11/15 16:17:11 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -50,6 +50,9 @@ extern const int db_value[];
 
 #define NODE_TRAVERSE     (1<<9)
 #define NODE_CHECKED      (1<<10)
-
+#define NODE_MOVED_OUT    (1<<11)
+#define NODE_MOVED_IN     (1<<12)
+#define NODE_ALLOW_NEW    (1<<13)
+#define NODE_ALLOW_RM	  (1<<14)
 
 #endif
