@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/db_config.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/db_config.h,v 1.12 2006/10/27 21:16:57 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,12 +20,12 @@
  
 #ifndef _DB_CONFIG_H_INCLUDED
 #define _DB_CONFIG_H_INCLUDED
-#include <sys/types.h>
+#include "aide.h"
+#include <unistd.h>
 #include <stdio.h>
 
 #define E2O(n) (1<<n)
 
-#include "config.h"
 #include "types.h"
 #include "list.h"
 #include "seltree.h"
@@ -35,6 +35,10 @@
 /* Warning! if acl in database is corrupted then
    this will break down. See and fix db.c */
 
+#ifndef WITH_ACL
+# error "No ACL support ... but Sun ACL support."
+#endif
+
 #include <sys/acl.h>
 typedef struct acl_type{
   int entries;
@@ -43,6 +47,41 @@ typedef struct acl_type{
 
 #endif
 
+#ifdef WITH_POSIX_ACL /* POSIX acl works for Sun ACL, AIUI but anyway... */
+#include <sys/acl.h>
+#ifndef WITH_ACL
+# error "No ACL support ... but POSIX ACL support."
+#endif
+#endif
+
+typedef struct acl_type {
+ char *acl_a; /* ACCESS */
+ char *acl_d; /* DEFAULT, directories only */
+} acl_type;
+
+#ifdef WITH_XATTR /* Do generic user Xattrs. */
+#include <sys/xattr.h>
+#include <attr/xattr.h>
+#endif
+
+typedef struct xattr_node 
+{
+ char *key;
+ byte *val;
+ size_t vsz;
+} xattr_node;
+
+typedef struct xattrs_type
+{
+  size_t num;
+  size_t sz;
+  struct xattr_node *ents;
+} xattrs_type;
+
+#ifdef WITH_SELINUX
+#include <selinux/selinux.h>
+#endif
+
 #ifdef WITH_MHASH
 #include <mhash.h>
 #endif
@@ -98,108 +137,81 @@ typedef enum {
    db_crc32b, 			/* "crc32b",  */
    db_attr,                     /* attributes */
    db_acl,                      /* access control list */
+   db_bsize,                    /* "bsize"    */
+   db_rdev,                     /* "rdev"     */
+   db_dev,                      /* "dev"      */
    db_checkmask,                /* "checkmask"*/
+   db_allownewfile,		/* "allownewfile */
+   db_allowrmfile,		/* "allowrmfile" */
+   db_sha256, 			/* "sha256",  */
+   db_sha512, 			/* "sha512",  */
+   db_whirlpool,		/* "whirlpool",  */
+   db_selinux, 			/* "selinux",  */
+   db_xattrs, 			/* "xattrs",  */
    db_unknown } DB_FIELD; 	/* "unknown"  */
 
 /* db_unknown must be last because it is used to determine size of
    DB_FILED */
 
+/* FIXME: THIS IS A HACK, sometimes we use AIDE_OFF_TYPE instead
+ * because that's what internal functions take. This bitmap needs to die. */
+#define DB_ATTR_TYPE unsigned long long
+#define DB_ATTR_UNDEF ((DB_ATTR_TYPE) -1)
+
 /* WE need this for rx_rules since enums are not orrable (horrible) */
-#define DB_FILENAME (1<<0)	/* "name",   */ 
-#define DB_LINKNAME (1<<1)	/* "lname",   */
-#define DB_PERM     (1<<2)	/* "perm",    */
-#define DB_UID      (1<<3)	/* "uid",     */
-#define DB_GID      (1<<4)	/* "gid",     */
-#define DB_SIZE     (1<<5)	/* "size",    */
-#define DB_ATIME    (1<<6)	/* "atime",   */
-#define DB_CTIME    (1<<7)	/* "ctime",   */
-#define DB_MTIME    (1<<8)	/* "mtime",   */
-#define DB_INODE    (1<<9)	/* "inode",   */
-#define DB_BCOUNT   (1<<10)	/* "bcount",  */
-#define DB_LNKCOUNT (1<<11)	/* "lcount",  */
-#define DB_MD5      (1<<12)	/* "md5",     */
-#define DB_SHA1     (1<<13)	/* "sha1",    */
-#define DB_RMD160   (1<<14)	/* "rmd160",  */
-#define DB_TIGER    (1<<15)	/* "tiger",   */
+#define DB_FILENAME (1LLU<<0)	/* "name",   */ 
+#define DB_LINKNAME (1LLU<<1)	/* "lname",   */
+#define DB_PERM     (1LLU<<2)	/* "perm",    */
+#define DB_UID      (1LLU<<3)	/* "uid",     */
+#define DB_GID      (1LLU<<4)	/* "gid",     */
+#define DB_SIZE     (1LLU<<5)	/* "size",    */
+#define DB_ATIME    (1LLU<<6)	/* "atime",   */
+#define DB_CTIME    (1LLU<<7)	/* "ctime",   */
+#define DB_MTIME    (1LLU<<8)	/* "mtime",   */
+#define DB_INODE    (1LLU<<9)	/* "inode",   */
+#define DB_BCOUNT   (1LLU<<10)	/* "bcount",  */
+#define DB_LNKCOUNT (1LLU<<11)	/* "lcount",  */
+#define DB_MD5      (1LLU<<12)	/* "md5",     */
+#define DB_SHA1     (1LLU<<13)	/* "sha1",    */
+#define DB_RMD160   (1LLU<<14)	/* "rmd160",  */
+#define DB_TIGER    (1LLU<<15)	/* "tiger",   */
 /*
   We want to matk these newertheless we have a 
   hash-functon or not.
  */
 
-#define DB_CRC32    (1<<16)	/* "crc32",   */
-#define DB_HAVAL    (1<<17)	/* "haval",   */
-#define DB_GOST     (1<<18)	/* "gost",    */
-#define DB_CRC32B   (1<<19)	/* "crc32b",  */
-#define DB_ACL      (1<<20)     /* "acl"      */
-#define DB_CHECKMASK (1<<21)    /* "checkmask"*/
-#define DB_SIZEG     (1<<22)	/* "unknown"  */
+#define DB_CRC32    (1LLU<<16)	/* "crc32",   */
+#define DB_HAVAL    (1LLU<<17)	/* "haval",   */
+#define DB_GOST     (1LLU<<18)	/* "gost",    */
+#define DB_CRC32B   (1LLU<<19)	/* "crc32b",  */
+// #define DB_ATTR    (1LLU<<20)     /* "attr"    */
+#define DB_ACL      (1LLU<<21)  /* "acl"      */
+#define DB_BSIZE    (1LLU<<22)  /* "bsize"    */
+#define DB_RDEV     (1LLU<<23)  /* "rdev"     */
+#define DB_DEV      (1LLU<<24)  /* "dev"      */
 
+#define DB_CHECKMASK  (1LLU<<25) /* "checkmask"*/
+#define DB_SIZEG      (1LLU<<26) /* "unknown"  */
+#define DB_CHECKINODE (1LLU<<27) /* "checkinode"*/
+#define DB_NEWFILE    (1LLU<<28) /* "allow new file" */
+#define DB_RMFILE     (1LLU<<29) /* "allot rm file" */
+#define DB_SHA256     (1LLU<<30) /* "sha256",  */
+#define DB_SHA512     (1LLU<<31) /* "sha512",  */
+#define DB_SELINUX    (1LLU<<32) /* "selinux", */
+#define DB_XATTRS     (1LLU<<33) /* "xattrs",  */
+#define DB_WHIRLPOOL  (1LLU<<34) /* "whirlpool",  */
 
 #define DB_HASHES    (DB_MD5|DB_SHA1|DB_RMD160|DB_TIGER|DB_CRC32|DB_HAVAL| \
-		      DB_GOST|DB_CRC32B)
+		      DB_GOST|DB_CRC32B|DB_SHA256|DB_SHA512|DB_WHIRLPOOL)
 
-const static char* db_names[] = {
-   "name", 
-   "lname", 
-   "perm", 
-   "uid", 
-   "gid", 
-   "size", 
-   "atime", 
-   "ctime",
-   "mtime", 
-   "inode", 
-   "bcount", 
-   "lcount", 
-   "md5", 
-   "sha1", 
-   "rmd160", 
-   "tiger",
-   "crc32", 
-   "haval",
-   "gost",
-   "crc32b",
-   "attr",
-   "acl",
-   "checkmask",
-   "unknown" } ; 
-
-const static int db_value[] = { 
-   db_filename, 	/* "name",   */ 
-   db_linkname, 	/* "lname",   */
-   db_perm, 		/* "perm",    */
-   db_uid, 		/* "uid",     */
-   db_gid,		/* "gid",     */
-   db_size, 		/* "size",    */
-   db_atime, 		/* "atime",   */
-   db_ctime, 		/* "ctime",   */
-   db_mtime, 		/* "mtime",   */
-   db_inode,		/* "inode",   */
-   db_bcount, 		/* "bcount",  */
-   db_lnkcount, 	/* "lcount",  */
-   db_md5, 		/* "md5",     */
-   db_sha1, 		/* "sha1",    */
-   db_rmd160,		/* "rmd160",  */
-   db_tiger, 		/* "tiger",   */
-   db_crc32, 		/* "crc32",   */
-   db_haval,		/* "haval",   */
-   db_gost, 		/* "gost",    */
-   db_crc32b, 		/* "crc32b",  */
-   db_attr,             /* attributes */
-   db_acl,              /* "acl"      */
-   db_checkmask,	/* "checkmask" */
-   db_unknown };	/* "unknown"  */
+extern const char* db_names[db_unknown+1];
+extern const int db_value[db_unknown+1];
 
 /* db_namealias && db_aliasvalue are here to support earlier database 
  * names that are no longer used. */
-
-const static char* db_namealias[] = {
-  "count" } ;
-
-const static int db_aliasvalue[] = {
-  db_lnkcount } ;       /* "count",  */
-
-const static int db_alias_size=1;
+#define db_alias_size 1
+extern const char* db_namealias[db_alias_size];
+extern const int db_aliasvalue[db_alias_size];
 
 /* TIMEBUFSIZE should be exactly ceil(sizeof(time_t)*8*ln(2)/ln(10))
  * Now it is ceil(sizeof(time_t)*2.5)
@@ -263,20 +275,21 @@ typedef struct db_config {
   char* config_file;
   char* config_version;
 
+ 
+  int do_dbnewmd;
+  int do_dboldmd; 
 #ifdef WITH_MHASH
   int do_configmd;
   MHASH confmd;
   hashid confhmactype;
   char* old_confmdstr;
 
-  int do_dbnewmd;
-  int do_dboldmd;
   hashid dbhmactype;
   MHASH dbnewmd;
   MHASH dboldmd;
+#endif
   char* old_dbnewmdstr;
   char* old_dboldmdstr;
-#endif
 
 
   /* The following three a lists of rx_rule*s */
@@ -316,7 +329,7 @@ typedef struct db_config {
   time_t end_time;
 
   int symlinks_found;
-  int attr;
+  DB_ATTR_TYPE attr;
 
 #ifdef WITH_ACL  
   int no_acl_on_symlinks;
@@ -332,15 +345,18 @@ typedef struct db_line {
   byte* sha1;
   byte* rmd160;
   byte* tiger;
-  byte* crc32;
+
+  byte* sha256;
+  byte* sha512;
+ 
+  byte* crc32; /* MHASH only */
   byte* haval;
   byte* gost;
   byte* crc32b;
+  byte* whirlpool; 
 
-#ifdef WITH_ACL
   acl_type* acl;
   /* Something here.. */
-#endif
 
   mode_t perm;
   mode_t perm_o; /* Permission for tree traverse */
@@ -358,8 +374,12 @@ typedef struct db_line {
   char* filename;
   char* linkname;
 
+  char *cntx;
+ 
+  xattrs_type* xattrs;
+
   /* Attributes .... */
-  int attr;
+  DB_ATTR_TYPE attr;
   
 } db_line;
 
