@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/db_file.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/db_file.h,v 1.3 2006/10/27 20:45:57 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +24,7 @@
 #include "db.h"
 
 extern void db_buff(int, FILE*);
-extern int db_scan(); /* Rumaa.... */
+extern int db_scan(void); /* Rumaa.... */
 extern char* dbtext; /* Todella rumaa... */
 extern long* db_lineno;
 extern long db_in_lineno;
@@ -33,7 +33,7 @@ extern long db_new_lineno;
 
 char** db_readline_file(int);
 int db_writespec_file(db_config*);
-int db_writeline_file(db_line* line,db_config* conf);
+int db_writeline_file(db_line* line,db_config* conf,url_t* url);
 int db_close_file(db_config* conf);
 #ifdef WITH_ZLIB
 void handle_gzipped_input(int out,gzFile*);
