@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 2002,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/md.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/md.h,v 1.5 2006/11/29 12:58:02 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,7 @@
 #ifndef _MD_H_INCLUDED
 #define _MD_H_INCLUDED
 
-#include "../config.h"
+#include "aide.h"
 
 /*
   This should come from configure-script.
@@ -32,13 +32,23 @@
 
 #ifdef WITH_MHASH
 #include <mhash.h>
-#define HASH_MHASH_COUNT 18
+
+#ifdef HAVE_MHASH_WHIRLPOOL
+# define HASH_MHASH_COUNT MHASH_WHIRLPOOL /* WHIRLPOOL == 22 on 2006-10-11 */
+#else
+# define HASH_MHASH_COUNT MHASH_RIPEMD320
+#endif
 #define MHASH_RMD160 MHASH_RIPEMD160
 #define MHASH_HAVAL MHASH_HAVAL256
 #endif
 
-#ifdef WITH_LIBGCRYPT
+#ifdef WITH_GCRYPT
 #include <gcrypt.h>
+#define HASH_GCRYPT_COUNT GCRY_MD_CRC32
+#ifndef WITH_MHASH
+#define HASH_USE_GCRYPT (DB_MD5|DB_SHA1|DB_RMD160|DB_TIGER|DB_CRC32|\
+                         DB_HAVAL|DB_CRC32|DB_SHA256|DB_SHA512)
+#endif
 #endif
 
 /*
@@ -59,6 +69,8 @@
 #define HASH_TIGER160_LEN 20
 #define HASH_MD4_LEN 16
 #define HASH_SHA256_LEN 32
+#define HASH_SHA512_LEN 64
+#define HASH_WHIRLPOOL_LEN 64
 #define HASH_ADLER32_LEN 4
 #define HASH_CRC32B_LEN 4
 #define HASH_CRC32_LEN 4
@@ -68,10 +80,13 @@
   What we use from what library?
  */
 
+#ifdef WITH_MHASH
 #define HASH_USE_MHASH (DB_MD5|DB_SHA1|DB_RMD160|DB_TIGER|DB_CRC32|\
-			DB_HAVAL|DB_GOST|DB_CRC32B)     
+                        DB_HAVAL|DB_GOST|DB_CRC32|\
+                        DB_SHA256|DB_SHA512|DB_WHIRLPOOL)
 
-#define HASH_USE_LIBGCRYPT (0)
+#define HASH_USE_GCRYPT (0)
+#endif
 
 
 /*
@@ -100,18 +115,20 @@ typedef struct md_container {
   char tiger160[HASH_TIGER160_LEN];
   char md4[HASH_MD4_LEN];
   char sha256[HASH_SHA256_LEN];
+  char sha512[HASH_SHA512_LEN];
   char adler32[HASH_ADLER32_LEN];
+  char whirlpool[HASH_WHIRLPOOL_LEN];
   
 
   /* 
      Attr which are to be calculated.
   */
-  int calc_attr; 
+  DB_ATTR_TYPE calc_attr; 
   /*
     Attr which are not (yet) to be calculated.
     After init hold's hashes which are not calculated :)
   */
-  int todo_attr;
+  DB_ATTR_TYPE todo_attr;
 
   /*
     Variables needed to cope with the library.
@@ -120,8 +137,8 @@ typedef struct md_container {
   MHASH mhash_mdh[HASH_MHASH_COUNT+1];
 #endif
 
-#ifdef WITH_LIBGCRYPT
-  GCRY_MD_HD mdh;
+#ifdef WITH_GCRYPT
+  gcry_md_hd_t mdh;
 #endif
 
 
