@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/symboltable.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/symboltable.h,v 1.2 2006/10/27 18:54:29 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,7 +26,7 @@
 typedef struct symba {
   char* name;
   char* value;
-  int ival;
+  DB_ATTR_TYPE ival;
 
 } symba;
 
