@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/url.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/url.h,v 1.2 2006/03/25 22:13:08 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +24,7 @@
 typedef enum {
   url_file, url_stdout, url_stdin, url_stderr, url_fd, url_http,
   url_sql, url_syslog, url_database, url_multiwriter, 
-  url_gzip, url_string, 
+  url_gzip, url_string, url_ftp, url_https, 
   url_unknown
 } URL_TYPE;
 
