@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/include/util.h,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/include/util.h,v 1.5 2006/11/24 20:40:21 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,10 +37,6 @@
 #  define stricmp(a,b)   strcasecmp( (a), (b) )
 #endif
 
-#ifndef HAVE_STPCPY
-char * stpcpy (char*,const char* );
-#endif
-
 int cmpurl(url_t*, url_t*);
 
 url_t* parse_url(char*);
@@ -57,6 +53,14 @@ void sig_handler(int signal);
 
 void init_sighandler(void);
 
+#ifndef HAVE_STRNSTR
 char* strnstr(char* haystack,char* needle,int n);
+#endif
+
+#ifndef HAVE_STRNLEN
+size_t strnlen(const char *s, size_t maxlen);
+#endif
+
+int syslog_facility_lookup(char *);
 
 #endif
