@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/aide.c,v 1.4 2003/11/28 15:03:58 rvdb Exp $
+ * $Header: /cvsroot/aide/aide/src/aide.c,v 1.26 2006/11/30 19:41:52 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
@@ -31,8 +33,6 @@
 #include <unistd.h>
 #endif
 
-#include "aide.h"
-
 #include "md.h"
 #include "commandconf.h"
 #include "compare_db.h"
@@ -50,33 +50,30 @@
 /*for locale support*/
 db_config* conf;
 
+#ifndef MAXHOSTNAMELEN
+#define MAXHOSTNAMELEN 256
+#endif
+
 void usage(int exitvalue)
 {
   fprintf(stderr, 
-	  _("Aide,\n"
-	    "Usage: aide <options> command\n"
-	    "\n"
+	  _("Aide " AIDEVERSION" \n\n"
+	    "Usage: aide [options] command\n\n"
 	    "Commands:\n"
-	    "-i\n--init\n\tInitialize the database\n"
-	    "-C\n--check\n\tCheck the database\n"
-	    "-u\n--update\n\t"
-	    "Check and update the database non-interactively.\n"
-	    "--compare\n\tCompare two databases\n"
-	    "-v\n--version\n\t"
-	    "Show version of AIDE and compilation options.\n"
-	    "-h\n--help\n\tShow this help message\n"
+	    "  -i, --init\t\tInitialize the database\n"
+	    "  -C, --check\t\tCheck the database\n"
+	    "  -u, --update\t\tCheck and update the database non-interactively\n"
+	    "      --compare\t\tCompare two databases\n\n"
+	    "Miscellaneous:\n"
+	    "      --config-check\tTest the configuration file\n"
+	    "  -v, --version\t\tShow version of AIDE and compilation options\n"
+	    "  -h, --help\t\tShow this help message\n\n"
 	    "Options:\n"
-	    "-c config_file\n--config=config_file\n\t"
-	    "Get config options from config_file.\n"
-	    "-B \"config_stuff\"\n--before=\"config_stuff\"\n\t"
-	    "Before config_file is read use these options.\n"
-	    "-A \"config_stuff\"\n--after=\"config_stuff\"\n\t"
-	    "After config_file is read use these options.\n"
-	    "-r reporter\n--report=reporter\n\t"
-	    "Where report output is written to.\n"
-	    "-Vverbosity_level\n--verbose=verbosity_level\n\t"
-	    "Level of debug messages.\n"
-	    "--config-check just read config-file\n"
+	    "  -c [cfgfile]\t--config=[cfgfile]\tGet config options from [cfgfile]\n"
+	    "  -B \"OPTION\"\t--before=\"OPTION\"\tBefore configuration file is read define OPTION\n"
+	    "  -A \"OPTION\"\t--after=\"OPTION\"\tAfter configuration file is read define OPTION\n"
+	    "  -r [reporter]\t--report=[reporter]\tWrite report output to [reporter] url\n"
+	    "  -V[level]\t--verbose=[level]\tSet debug message level to [level]\n"
 	    "\n")
 	  );
   
@@ -86,8 +83,8 @@ void usage(int exitvalue)
 void print_version(void)
 {
   fprintf(stderr,
-	  "Aide, version " AIDEVERSION "\n"
-	  "Compiled with the following options\n" AIDECOMPILEOPTIONS "\n");
+	  "Aide " AIDEVERSION "\n\n"
+	  "Compiled with the following options:\n\n" AIDECOMPILEOPTIONS "\n");
   exit(0);
 }
 
@@ -112,7 +109,7 @@ int read_param(int argc,char**argv)
     { "update", no_argument, NULL, 'u'},
     { "config-check", no_argument, NULL, 'D'},
     { "compare", no_argument, NULL, 'E'},
-    { NULL }
+    { NULL,0,NULL,0 }
   };
 
   while(1){
@@ -248,8 +245,22 @@ void setdefaults_before_config()
 {
   char* urlstr=INITIALERRORSTO;
   url_t* u=NULL;
+  char* s=(char*)malloc(sizeof(char)*MAXHOSTNAMELEN+1);
+  unsigned long long p;
 
+  /*
+    Set up the hostname
+  */
   conf=(db_config*)malloc(sizeof(db_config));
+  conf->defsyms=NULL;
+  
+  if (gethostname(s,MAXHOSTNAMELEN)==-1) {
+    error(0,_("Couldn't get hostname"));
+    free(s);
+  } else {
+    s=(char*)realloc((void*)s,strlen(s)+1);
+    do_define("HOSTNAME",s);
+  }
   
   /* Setting some defaults */
   conf->report_db=0;  
@@ -266,19 +277,27 @@ void setdefaults_before_config()
   error_init(u,1);
 
   conf->config_file=CONFIG_FILE;
+  conf->config_version=NULL;
+  
+#ifdef WITH_ACL
+  conf->no_acl_on_symlinks=0; /* zero means don't do ACLs on symlinks */
+#endif
+  
 #ifdef WITH_MHASH
   conf->do_configmd=0;
   conf->confmd=NULL;
   conf->confhmactype=CONFIGHMACTYPE;
   conf->old_confmdstr=NULL;
-  conf->do_dbnewmd=0;
-  conf->do_dboldmd=0;
   conf->dbhmactype=DBHMACTYPE;
-  conf->old_dbnewmdstr=NULL;
-  conf->old_dboldmdstr=NULL;
   conf->dbnewmd=NULL;
   conf->dboldmd=NULL;
 #endif
+  
+  conf->do_dbnewmd=0;
+  conf->do_dboldmd=0;
+  conf->old_dbnewmdstr=NULL;
+  conf->old_dboldmdstr=NULL;
+  
   conf->db_out_order=(DB_FIELD*)malloc(sizeof(DB_FIELD)*db_unknown);
   conf->db_out_size=1;
   conf->db_out_order[0]=db_filename;
@@ -293,10 +312,12 @@ void setdefaults_before_config()
   conf->db_new=NULL;
   conf->db_out_url=NULL;
   conf->db_out=NULL;
+  
 #ifdef WITH_ZLIB
   conf->db_gzin=0;
   conf->db_gznew=0;
   conf->gzip_dbout=0;
+  conf->db_gzout=0;
 #endif
 
   conf->action=0;
@@ -308,47 +329,94 @@ void setdefaults_before_config()
   conf->equrxlst=NULL;
   conf->negrxlst=NULL;
 
-  conf->defsyms=NULL;
   conf->groupsyms=NULL;
 
   conf->start_time=time(&(conf->start_time));
 
-  
+  do_groupdef("ANF",DB_NEWFILE);
+  do_groupdef("ARF",DB_RMFILE);
   do_groupdef("p",DB_PERM);
   do_groupdef("i",DB_INODE);
+  do_groupdef("I",DB_CHECKINODE);
   do_groupdef("n",DB_LNKCOUNT);
   do_groupdef("u",DB_UID);
   do_groupdef("g",DB_GID);
+  do_groupdef("l",DB_LINKNAME);
   do_groupdef("s",DB_SIZE);
   do_groupdef("S",DB_SIZEG);
   do_groupdef("b",DB_BCOUNT);
   do_groupdef("m",DB_MTIME);
   do_groupdef("c",DB_CTIME);
   do_groupdef("a",DB_ATIME);
+#if defined(WITH_MHASH) || defined(WITH_GCRYPT)
   do_groupdef("md5",DB_MD5);
   do_groupdef("tiger",DB_TIGER);
+  do_groupdef("haval",DB_HAVAL);
+  do_groupdef("crc32",DB_CRC32);
   do_groupdef("sha1",DB_SHA1);
   do_groupdef("rmd160",DB_RMD160);
+  do_groupdef("sha256",DB_SHA256);
+  do_groupdef("sha512",DB_SHA512);
+#endif
 #ifdef WITH_ACL
   do_groupdef("acl",DB_ACL);
 #endif
-#ifdef WITH_MHASH
-  do_groupdef("crc32",DB_CRC32);
-  /*
-    crc32b is same than crc32
-    do_groupdef("crc32b",DB_CRC32B);
-  */
-  do_groupdef("haval",DB_HAVAL);
-  do_groupdef("gost",DB_GOST);
+#ifdef WITH_XATTR
+  do_groupdef("xattrs",DB_XATTRS);
+#endif
+#ifdef WITH_SELINUX
+  do_groupdef("selinux",DB_SELINUX);
 #endif
 
+#ifdef WITH_MHASH
+  do_groupdef("gost",DB_GOST);
+#ifdef HAVE_MHASH_WHIRLPOOL
+  do_groupdef("whirlpool",DB_WHIRLPOOL);
+#endif
+#endif
+
+  p=0LLU;
+#if defined(WITH_MHASH) || defined(WITH_GCRYPT)
+  p|=DB_MD5;
+#endif
+#ifdef WITH_ACL
+  p|=DB_ACL;
+#endif
+#ifdef WITH_SELINUX
+  p|=DB_SELINUX;
+#endif
+#ifdef WITH_XATTR
+  p|=DB_XATTRS;
+#endif
   do_groupdef("R",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZE|
-	      DB_MTIME|DB_CTIME|DB_MD5);
-  do_groupdef("L",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID);
-  do_groupdef(">",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZEG);
+                  DB_LINKNAME|DB_MTIME|DB_CTIME|p);
+
+  p=0LLU;
+#ifdef WITH_ACL
+  p|=DB_ACL;
+#endif
+#ifdef WITH_SELINUX
+  p|=DB_SELINUX;
+#endif
+#ifdef WITH_XATTR
+  p|=DB_XATTRS;
+#endif
+  do_groupdef("L",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_LINKNAME|p);
+
+  p=0LLU;
+#ifdef WITH_ACL
+  p|=DB_ACL;
+#endif
+#ifdef WITH_SELINUX
+  p|=DB_SELINUX;
+#endif
+#ifdef WITH_XATTR
+  p|=DB_XATTRS;
+#endif
+  do_groupdef(">",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZEG|
+		  DB_LINKNAME|p);
   do_groupdef("E",0);
-  
-  
+
 }
 
 void setdefaults_after_config()
@@ -401,12 +469,6 @@ int main(int argc,char**argv)
 
   setdefaults_before_config();
 
-  if(argc<=1){
-    error(0,_("No options given. Exiting...\n"));
-    usage(INVALID_ARGUMENT_ERROR);
-  }
-  
-  
   if(read_param(argc,argv)==RETFAIL){
     error(0, _("Invalid argument\n") );
     exit(INVALID_ARGUMENT_ERROR);
@@ -429,28 +491,9 @@ int main(int argc,char**argv)
   
   conf->tree=gen_tree(conf->selrxlst,conf->negrxlst,conf->equrxlst);
   
-  
-  /*
-    Just for debugging purposes..
-   */
-  /*
-  db_disk_init();
-  {
-    int i=0;
-    db_line* l=db_readline_disk();
-    printf("Entering to readloop\n");
-    while (l!=NULL) {
-      i++;
-      printf("%i:%s\n",i,l->filename);
-      l=db_readline_disk();
-    }
-    printf("Readloop ended\n");
-    exit(0);
-  }  
-  */  
   /* Let's do some sanity checks for the config */
   if(cmpurl(conf->db_in_url,conf->db_out_url)==RETOK){
-    error(5,_("WARNING:Input and output database urls are the same.\n"));
+    error(4,_("WARNING:Input and output database urls are the same.\n"));
     if((conf->action&DO_INIT)&&(conf->action&DO_COMPARE)){
       error(0,_("Input and output database urls cannot be the same "
 	    "when doing database update\n"));
@@ -499,10 +542,9 @@ int main(int argc,char**argv)
 #endif
   conf->use_initial_errorsto=0;
   if (!conf->config_check) {
-  
+    int exitcode=0;
     long totalchanges=0;
     long stat[5]={1,0,0,0,0};
-    conf->tree=gen_tree(conf->selrxlst,conf->negrxlst,conf->equrxlst);
     if(conf->action&DO_INIT){
       if(db_init(DB_WRITE)==RETFAIL) {
 	exit(IO_ERROR);
@@ -529,10 +571,12 @@ int main(int argc,char**argv)
     }
       
     populate_tree(conf->tree);
+    db_close(conf);
 
     /* First terse report */
     totalchanges=report_tree(conf->tree,0,stat);
     if(totalchanges>0){
+      exitcode=(stat[2]!=0)*1+(stat[3]!=0)*2+(stat[4]!=0)*4;
       stat[0]=1;
       report_tree(conf->tree,1,stat);
       stat[0]=1;
@@ -543,18 +587,22 @@ int main(int argc,char**argv)
       stat[0]=1;
       report_tree(conf->tree,4,stat);
     } else {
-      printf("\nAIDE, version " AIDEVERSION "\n\n");
-      if(conf->action&DO_COMPARE) {
-        printf("### All files match AIDE database.  Looks okay!\n\n");
-      }
-      if(conf->action&DO_INIT) {
-        printf("### AIDE database initialized.\n\n");
+      if (conf->verbose_level >= 5) {
+        printf("\nAIDE, version " AIDEVERSION "\n\n");
+        if(conf->action&DO_COMPARE) {
+          printf("### All files match AIDE database. Looks okay!\n\n");
+        }
+        if(conf->action&DO_INIT) {
+	  if(conf->action&DO_COMPARE)
+            printf("### New AIDE database written to %s\n\n",conf->db_out_url->value);
+	  else
+            printf("### AIDE database at %s initialized.\n\n",conf->db_out_url->value);
+        }
       }
     }
-    db_close(conf);
     
-    exit(RETOK);
-      
+    exit(exitcode);
+    
   }else {
 #ifdef WITH_MHASH
     if(conf->confmd){
@@ -575,3 +623,4 @@ int main(int argc,char**argv)
 const char* aide_key_3=CONFHMACKEY_03;
 const char* db_key_3=DBHMACKEY_03;
 
+// vi: ts=8 sw=8
