@@ -2,7 +2,7 @@
 **
 ** Copyright (C) 1994 Swedish University Network (SUNET)
 ** Modified by Rami Lehti (C) 1999
-** $Header: /cvsroot/aide/aide/src/base64.c,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+** $Header: /cvsroot/aide/aide/src/base64.c,v 1.6 2006/10/27 20:42:20 rvdb Exp $
 **
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -30,10 +30,10 @@
 **
 */
 
+#include "aide.h"
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "config.h"
 #include "base64.h"
 #include "report.h"
 /*for locale support*/
@@ -103,8 +103,6 @@ char* encode_base64(byte* src,size_t ssize)
   }
   outbuf = (char *)malloc(sizeof(char)*B64_BUF);
   
-  memset((void*)outbuf,0,B64_BUF);
-  
   /* Initialize working pointers */
   inb = src;
   i = 0;
@@ -174,7 +172,7 @@ char* encode_base64(byte* src,size_t ssize)
 }
 
 /* FIXME Possible buffer overflow on outputs larger than B64_BUF */
-byte* decode_base64(char* src,size_t ssize)
+byte* decode_base64(char* src,size_t ssize, size_t *ret_len)
 {
   byte* outbuf;
   byte* retbuf;
@@ -191,11 +189,9 @@ byte* decode_base64(char* src,size_t ssize)
     return NULL;
 
 
-
   /* Initialize working pointers */
   inb = src;
   outbuf = (byte *)malloc(sizeof(byte)*B64_BUF);
-  memset(outbuf,0,sizeof(byte)*B64_BUF);
 
   l = 0;
   triple = 0;
@@ -211,7 +207,7 @@ byte* decode_base64(char* src,size_t ssize)
       switch(i)
 	{
 	case FAIL:
-	  error(5, "decode_base64: Illegal character: %c\n", *inb);
+	  error(3, "decode_base64: Illegal character: %c\n", *inb);
 	  error(230, "decode_base64: Illegal line:\n%s\n", src);
 	  return NULL;
 	  break;
@@ -252,6 +248,8 @@ byte* decode_base64(char* src,size_t ssize)
   
   free(outbuf);
 
+  if (ret_len) *ret_len = pos;
+  
   return retbuf;
 }
 
@@ -267,7 +265,7 @@ size_t length_base64(char* src,size_t ssize)
   error(210, "decode base64\n");
   /* Exit on empty input */
   if (!ssize||src==NULL)
-    return NULL;
+    return 0;
 
 
 
@@ -288,9 +286,9 @@ size_t length_base64(char* src,size_t ssize)
       switch(i)
 	{
 	case FAIL:
-	  error(5, "length_base64: Illegal character: %c\n", *inb);
+	  error(3, "length_base64: Illegal character: %c\n", *inb);
 	  error(230, "length_base64: Illegal line:\n%s\n", src);
-	  return NULL;
+	  return 0; 
 	  break;
 	case SKIP:
 	  break;
