@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/be.c,v 1.2 2003/08/18 13:06:30 rvdb Exp $
+ * $Header: /cvsroot/aide/aide/src/be.c,v 1.10 2006/12/14 16:09:24 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,16 +18,18 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <unistd.h>
 #include <errno.h>
-#include "aide.h"
 #include "db_config.h"
 #include "db_file.h"
 #include "report.h"
+#include "fopen.h"
+#include "be.h"
 
 #ifdef WITH_PSQL
 #include "libpq-fe.h"
@@ -51,12 +53,6 @@ int be_sql_readinit(psql_data* ret) {
   s=strcat(s,ret->table);
   
   ret->res=PQexec(ret->conn,s);
-  /*
-  PQclear(ret->res);
-  ret->res=PQexec(ret->conn,
-		  ret->table
-		  );
-  */
 		  
   if (!ret->res || PQresultStatus(ret->res) != PGRES_COMMAND_OK) {
     
@@ -108,11 +104,6 @@ char* get_first_value(char** in){
   while((*in)[i]!=':' && (*in)[i]!='\0') {
     i++;
   }
-  /*  
-      if (i==0) {
-      return NULL;
-      }
-  */
   if ((*in)[i]!='\0') { /* Lets not go beond the sting.. */
     (*in)[i]='\0';
     (*in)+=i+1;
@@ -122,11 +113,15 @@ char* get_first_value(char** in){
 
 #endif
 
-void* be_init(int inout,url_t* u,int iszipped)
+FILE* be_init(int inout,url_t* u,int iszipped)
 {
-  void* fh=NULL;
+  FILE* fh=NULL;
   long a=0;
   char* err=NULL;
+  int fd;
+#if HAVE_FCNTL && HAVE_FTRUNCATE
+  struct flock fl;
+#endif
 
   if (u==NULL) {
     return NULL;
@@ -135,9 +130,38 @@ void* be_init(int inout,url_t* u,int iszipped)
   switch (u->type) {
   case url_file : {
     error(200,_("Opening file \"%s\" for %s\n"),u->value,inout?"r":"w+");
+#if HAVE_FCNTL && HAVE_FTRUNCATE
+    fd=open(u->value,inout?O_RDONLY:O_CREAT|O_RDWR,0666);
+#else
+    fd=open(u->value,inout?O_RDONLY:O_CREAT|O_RDWR|O_TRUNC,0666);
+#endif
+    error(255,"Opened file \"%s\" with fd=%i\n",u->value,fd);
+    if(fd==-1) {
+      error(0,_("Couldn't open file %s for %s"),u->value,
+	    inout?"reading\n":"writing\n");
+      return NULL;
+    }
+#if HAVE_FCNTL && HAVE_FTRUNCATE
+    if(!inout) {
+      fl.l_type = F_WRLCK;
+      fl.l_whence = SEEK_SET;
+      fl.l_start = 0;
+      fl.l_len = 0;
+      if (fcntl(fd, F_SETLK, &fl) == -1) {
+	if (fcntl(fd, F_SETLK, &fl) == -1)
+	  error(0,_("File %s is locked by another process.\n"),u->value);
+	else
+	  error(0,_("Cannot get lock for file %s"),u->value);
+	return NULL;
+      }
+      if(ftruncate(fd,0)==-1)
+	error(0,_("Error truncating file %s"),u->value);
+
+    }
+#endif
 #ifdef WITH_ZLIB
     if(iszipped && !inout){
-      fh=gzopen(u->value,"wb9+");
+      fh=gzdopen(fd,"wb9+");
       if(fh==NULL){
 	error(0,_("Couldn't open file %s for %s"),u->value,
 	      inout?"reading\n":"writing\n");
@@ -145,7 +169,7 @@ void* be_init(int inout,url_t* u,int iszipped)
     }
     else{
 #endif
-      fh=fopen(u->value,inout?"r":"w+");
+      fh=fdopen(fd,inout?"r":"w+");
       if(fh==NULL){
 	error(0,_("Couldn't open file %s for %s"),u->value,
 	      inout?"reading\n":"writing\n");
@@ -280,9 +304,6 @@ void* be_init(int inout,url_t* u,int iszipped)
       }
     }
    
-    /*printf("host:\"%s\" port:\"%s\" database:\"%s\" login:\"%s\" passwd:\"%s\" table:\"%s\"\n",pghost,pgport,dbName,login,pwd,ret->table); */
-    
-    
     if (login[0] == '\0' ) {
       login = NULL;
     }
@@ -322,9 +343,20 @@ void* be_init(int inout,url_t* u,int iszipped)
     return ret;
   }
 #endif
-
+#ifdef WITH_CURL
+  case url_http:
+  case url_https:
+  case url_ftp:
+    {
+      error(200,_("Opening curl \"%s\" for %s\n"),u->value,inout?"r":"w+");
+      if (iszipped) {
+	return NULL;
+      }
+      return url_fopen(u->value,inout?"r":"w+");
+    }
+#endif /* WITH CURL */
   default:{
-    error(0,"Unsupported backend:%i", u->type);
+    error(0,"Unsupported backend: %i", u->type);
     return NULL;
   }    
   }
