@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/commandconf.c,v 1.2 2003/11/04 15:39:48 rvdb Exp $
+ * $Header: /cvsroot/aide/aide/src/commandconf.c,v 1.19 2006/12/14 16:09:24 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,7 +27,6 @@
 #include <sys/param.h>
 
 #include "commandconf.h"
-#include "aide.h"
 #include "conf_lex.h"
 #include "conf_yacc.h"
 #include "db.h"
@@ -38,7 +38,7 @@
 /*for locale support*/
 #include "locale-aide.h"
 /*for locale support*/
-
+#include "fopen.h"
 
 #define BUFSIZE 4096
 #define ZBUFSIZE 16384
@@ -220,6 +220,7 @@ int conf_input_wrapper(char* buf, int max_size, FILE* in)
 	}
       } else {
 	conf->do_configmd=0;
+	return retval;
       }
     }
     /* FIXME This does not handle the case that @@end_config is on 
@@ -230,8 +231,8 @@ int conf_input_wrapper(char* buf, int max_size, FILE* in)
     } else {
       mhash(conf->confmd,(void*)buf,retval);
     }
-#endif
   }
+#endif
   return retval;
 }
 
@@ -239,10 +240,11 @@ int db_input_wrapper(char* buf, int max_size, int db)
 {
   int retval=0;
   int c=0;
-  char* tmp=NULL;
   int err=0;
-  int* domd=0;
+  int* domd=NULL;
+  url_t* db_url=NULL;
 #ifdef WITH_MHASH
+  char* tmp=NULL;
   MHASH* md=NULL;
   void* key=NULL;
   int keylen;
@@ -251,21 +253,32 @@ int db_input_wrapper(char* buf, int max_size, int db)
 #ifdef WITH_ZLIB
   gzFile* db_gzp=NULL;
 #endif
- 
   switch(db) {
   case DB_OLD: {
+    db_url=conf->db_in_url;
+    
     domd=&(conf->do_dboldmd);
+#ifdef WITH_MHASH
     md=&(conf->dboldmd);
+#endif
+    
     db_filep=&(conf->db_in);
+    
 #ifdef WITH_ZLIB
     db_gzp=&(conf->db_gzin);
 #endif
     break;
   }
   case DB_NEW: {
+    db_url=conf->db_new_url;
+    
     domd=&(conf->do_dbnewmd);
+#ifdef WITH_MHASH
     md=&(conf->dbnewmd);
+#endif
+    
     db_filep=&(conf->db_new);
+    
 #ifdef WITH_ZLIB
     db_gzp=&(conf->db_gznew);
 #endif
@@ -273,6 +286,18 @@ int db_input_wrapper(char* buf, int max_size, int db)
   }
   }
 
+#ifdef WITH_CURL
+  switch (db_url->type) {
+  case url_http:
+  case url_https:
+  case url_ftp: {
+    retval=url_fread(buf,1,max_size,(URL_FILE *)*db_filep);
+    break;
+  } 
+  default:
+#endif /* WITH CURL */
+
+
   /* Read a character at a time until we are doing md */
 #ifdef WITH_ZLIB
   if((*db_gzp==NULL)&&(*domd)){
@@ -284,16 +309,16 @@ int db_input_wrapper(char* buf, int max_size, int db)
       buf[0]='\0';
     }else {
       if((retval=gzread(*db_gzp,buf,max_size))<0){
-	error(0,_("gzread() failed:gzerr=%s!\n"),gzerror(*db_gzp,&err));
+	error(0,_("gzread() failed: gzerr=%s!\n"),gzerror(*db_gzp,&err));
 	retval=0;
 	buf[0]='\0';
       } else {
 	/* gzread returns 0 even if uncompressed bytes were read*/
 	error(240,"nread=%d,strlen(buf)=%d,errno=%s,gzerr=%s\n",
-		retval,strlen((char*)buf),strerror(errno),
+              retval,strnlen((char*)buf, max_size),strerror(errno),
 	      gzerror(*db_gzp,&err));
 	if(retval==0){
-	  retval=strlen((char*)buf);
+	  retval=strnlen((char*)buf, max_size);
 	}
       }
     }
@@ -309,9 +334,15 @@ int db_input_wrapper(char* buf, int max_size, int db)
       if(c==(unsigned char)'\213'){
 	/* We got gzip header. */
 	error(255,"Got Gzip header. Handling..\n");
-	fseek(*db_filep,0L,SEEK_SET);
+	lseek(fileno(*db_filep),0L,SEEK_SET);
 	*db_gzp=gzdopen(fileno(*db_filep),"rb");
 	c=gzgetc(*db_gzp);
+	error(255,"First character after gzip header is: %c(%#X)\n",c,c);
+  if(c==-1) {
+    int xx;
+	  error(0,"Error reading gzipped file: %s\n",gzerror(*db_gzp,&xx));
+    abort();
+  }
       }else {
 	/* False alarm */
 	ungetc(c,*db_filep);
@@ -333,7 +364,6 @@ int db_input_wrapper(char* buf, int max_size, int db)
 #endif /* WITH_MHASH */ 
 #endif /* WITH_ZLIB */
 
-
 #ifdef WITH_MHASH    
   if(*domd){
     if(!*md){
@@ -366,14 +396,18 @@ int db_input_wrapper(char* buf, int max_size, int db)
 	mhash(*md,(void*)buf,retval);
       }
     }
-#endif
   }
+#endif
+
+#ifdef WITH_CURL
+  }
+#endif /* WITH CURL */
   return retval;
 }
 
 int check_db_order(DB_FIELD* d,int size, DB_FIELD a)
 {
-  int i=0;
+  int i;
   for(i=0;i<size;i++){
     if(d[i]==a)
       return RETFAIL;
@@ -385,7 +419,7 @@ int check_dboo(DB_FIELD a){
   return check_db_order(conf->db_out_order,conf->db_out_size,a);
 }
 
-void update_db_out_order(int attr)
+void update_db_out_order(DB_ATTR_TYPE attr)
 {
   /* First we add those attributes that must be there */
   if (check_dboo(db_linkname)==RETOK) {
@@ -421,7 +455,7 @@ void update_db_out_order(int attr)
   if((attr&DB_CTIME) && (check_dboo(db_ctime)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_ctime;
   }
-  if((attr&DB_INODE) && (check_dboo(db_inode)!=RETFAIL)){
+  if((attr&DB_INODE||attr&DB_CHECKINODE) && (check_dboo(db_inode)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_inode;
   }
   if((attr&DB_LNKCOUNT) && (check_dboo(db_lnkcount)!=RETFAIL)){
@@ -439,29 +473,44 @@ void update_db_out_order(int attr)
   if((attr&DB_TIGER) && (check_dboo(db_tiger)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_tiger;
   }
-  /*
-#ifdef WITH_MHASH
-  */
   if((attr&DB_CRC32) && (check_dboo(db_crc32)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_crc32;
   }
   if((attr&DB_HAVAL) && (check_dboo(db_haval)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_haval;
   }
+  /*
+#ifdef WITH_MHASH
+  */
   if((attr&DB_GOST) && (check_dboo(db_gost)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_gost;
   }
+  if((attr&DB_WHIRLPOOL) && (check_dboo(db_whirlpool)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_whirlpool;
+  }
   if((attr&DB_CRC32B) && (check_dboo(db_crc32b)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_crc32b;
   }
   /*
 #endif
   */
+  if((attr&DB_SHA256) && (check_dboo(db_sha256)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_sha256;
+  }
+  if((attr&DB_SHA512) && (check_dboo(db_sha512)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_sha512;
+  }
 #ifdef WITH_ACL
   if((attr&DB_ACL) && (check_dboo(db_acl)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_acl;
   }
 #endif
+  if((attr&DB_XATTRS) && (check_dboo(db_xattrs)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_xattrs;
+  }
+  if((attr&DB_SELINUX) && (check_dboo(db_selinux)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_selinux;
+  }
   if((attr&DB_CHECKMASK) && (check_dboo(db_checkmask)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_checkmask;
   }
@@ -638,14 +687,24 @@ int do_ifxhost(int mode,char* name)
 {
   int doit;
   char* s=NULL;
+  char *p;
   doit=mode;
   s=(char*)malloc(sizeof(char)*MAXHOSTNAMELEN+1);
+  if (s == NULL) {
+    error(0,_("Couldn't malloc hostname buffer"));
+  }
+  s[MAXHOSTNAMELEN] = '\0';
 
   if(gethostname(s,MAXHOSTNAMELEN)==-1){
     error(0,_("Couldn't get hostname %s"),name);
     free(s);
     return -1;
   }  
+  /* strip off everything past the . */
+  p = strchr(s, '.');
+  if (p != NULL) {
+    *p = '\0';
+  }
   if(strcmp(name,s)==0) {
     doit=1-doit;
   }
@@ -653,7 +712,7 @@ int do_ifxhost(int mode,char* name)
   return (handle_endif(doit,1));
 }
 
-list* append_rxlist(char* rx,int attr,list* rxlst)
+list* append_rxlist(char* rx,DB_ATTR_TYPE attr,list* rxlst)
 {
   extern long conf_lineno; /* defined & set in conf_lex.l */
     
@@ -662,13 +721,15 @@ list* append_rxlist(char* rx,int attr,list* rxlst)
   r->rx=rx;
   r->attr=attr;
   r->conf_lineno = conf_lineno;
+  if (attr&DB_CHECKINODE && attr&DB_CTIME)
+    error(20,"Rule at line %i has c and I flags enabled at the same time. If same inode is found, flag c is ignored\n",conf_lineno);
   update_db_out_order(r->attr);
   rxlst=list_append(rxlst,(void*)r);
   
   return rxlst;
 }
 
-void do_groupdef(char* group,int value)
+void do_groupdef(char* group,DB_ATTR_TYPE value)
 {
   list* r=NULL;
   symba* s=NULL;
@@ -684,22 +745,23 @@ void do_groupdef(char* group,int value)
   conf->groupsyms=list_append(conf->groupsyms,(void*)s);
 }
 
-int get_groupval(char* group)
+DB_ATTR_TYPE get_groupval(char* group)
 {
   list* r=NULL;
+
   if((r=list_find(group,conf->groupsyms))){
     return (((symba*)r->data)->ival);
   }
   return -1;
 }
 
-
-
 void do_dbdef(int dbtype,char* val)
 {
   url_t* u=NULL;
   url_t** conf_db_url;
 
+  error(255,"do_dbdef (%i) called with (%s)\n",dbtype,val);
+
   switch(dbtype) {
   case DB_OLD: {
     conf_db_url=&(conf->db_in_url);
@@ -834,7 +896,7 @@ void do_verbdef(char* val)
 const char* aide_key_7=CONFHMACKEY_07;
 const char* db_key_7=DBHMACKEY_07;
 
-void* get_conf_key() {
+void* get_conf_key(void) {
   void* r;
   char* m=(char*)malloc(strlen(aide_key_1)+
 			strlen(aide_key_2)+
@@ -858,14 +920,14 @@ void* get_conf_key() {
   strcat(m,aide_key_8);
   strcat(m,aide_key_9);
   
-  r=decode_base64(m,strlen(m));
+  r=decode_base64(m,strlen(m),NULL);
 
   memset(m,0,strlen(m));
   free(m);
   return r;
 }
 
-size_t get_conf_key_len() {
+size_t get_conf_key_len(void) {
   size_t len=0;
   char* m=(char*)malloc(strlen(aide_key_1)+
 			strlen(aide_key_2)+
@@ -896,7 +958,7 @@ size_t get_conf_key_len() {
   return len;
 }
 
-void* get_db_key() {
+void* get_db_key(void) {
   void* r;
   char* m=(char*)malloc(strlen(db_key_1)+
 			strlen(db_key_2)+
@@ -920,14 +982,14 @@ void* get_db_key() {
   strcat(m,db_key_8);
   strcat(m,db_key_9);
   
-  r=decode_base64(m,strlen(m));
+  r=decode_base64(m,strlen(m),NULL);
   
   memset(m,0,strlen(m));
   free(m);
   return r;
 }
 
-size_t get_db_key_len() {
+size_t get_db_key_len(void) {
   size_t len=0;
   char* m=(char*)malloc(strlen(db_key_1)+
 			strlen(db_key_2)+
