@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/compare_db.c,v 1.2 2003/11/04 15:39:48 rvdb Exp $
+ * Copyright (C) 1999-2006 Rami Lehti, Pablo Virolainen, Richard van den Berg
+ * $Id: compare_db.c,v 1.19 2006/10/27 20:44:38 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,15 +18,22 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <sys/stat.h>
+#include <math.h>
+#ifdef WITH_AUDIT
+#include <libaudit.h>
+#ifdef HAVE_SYSLOG
+#include <syslog.h>
+#endif
+#endif
 
 #include "base64.h"
 #include "report.h"
 #include "db_config.h"
-#include "aide.h"
 #include "gnu_regex.h"
 #include "gen_list.h"
 #include "list.h"
@@ -34,6 +41,7 @@
 #include "util.h"
 #include "commandconf.h"
 #include "gen_list.h"
+#include "compare_db.h"
 /*for locale support*/
 #include "locale-aide.h"
 /*for locale support*/
@@ -45,13 +53,34 @@
 const int old_col  = 12;   
 const int new_col  = 40;   
 
-const int part_len = 40; /* usable length of line[] */
-char      oline[40];
-char      nline[40];
-const char* entry_format="  %-9s: %-34s, %-34s\n";
+const int part_len = 33; /* usable length of line[] */
+char      oline[33];
+char      nline[33];
+const char* entry_format=        "  %-9s: %-33s, %s\n";
+const char* entry_format_justnew="  %-9s: %-33c  %s\n";
 /*************/
 
+static int get_ignorelist() {
+  int ignorelist;
+  ignorelist=get_groupval("ignore_list");
 
+  if (ignorelist==-1) {
+    ignorelist=0;
+  }
+
+  return ignorelist;
+}
+
+static int get_report_attributes() {
+  int forced_attrs;
+  
+  forced_attrs=get_groupval("report_attributes");
+  if (forced_attrs==-1) {
+    forced_attrs=0;
+  }
+
+  return forced_attrs;
+}
 
 list* find_line_match(db_line* line,list* l)
 {
@@ -78,8 +107,34 @@ list* find_line_match(db_line* line,list* l)
   return NULL;
 }
 
-#ifdef WITH_ACL
-int compare_single_acl(aclent_t* a1,aclent_t* a2) {
+#ifdef WITH_POSIX_ACL
+int compare_acl(acl_type* a1,acl_type* a2)
+{
+  if (a1==NULL && a2==NULL) {
+    return RETOK;
+  }
+  if (a1==NULL || a2==NULL) {
+    return RETFAIL;
+  }
+
+  if (!a1->acl_a != !a2->acl_a) {
+    return RETFAIL;
+  }
+  if (!a1->acl_d != !a2->acl_d) {
+    return RETFAIL;
+  }
+
+  if (a1->acl_a && strcmp(a1->acl_a, a2->acl_a))
+    return RETFAIL;
+  if (a1->acl_d && strcmp(a1->acl_d, a2->acl_d))
+    return RETFAIL;
+
+  return RETOK;
+}
+#endif
+
+#ifdef WITH_SUN_ACL
+static int compare_single_acl(aclent_t* a1,aclent_t* a2) {
   if (a1->a_type!=a2->a_type ||
       a1->a_id!=a2->a_id ||
       a1->a_perm!=a2->a_perm) {
@@ -113,13 +168,73 @@ int compare_acl(acl_type* a1,acl_type* a2) {
 }
 #endif
 
+static int cmp_xattr_node(const void *c1, const void *c2)
+{
+  const xattr_node *x1 = c1;
+  const xattr_node *x2 = c2;
+
+  return (strcmp(x1->key, x2->key));
+}
+
+int compare_xattrs(xattrs_type* x1,xattrs_type* x2)
+{
+  size_t num = 0;
+
+  if (x1 && (x1->num == 0)) x1 = NULL;
+  if (x2 && (x2->num == 0)) x2 = NULL;
+
+  if (x1==NULL && x2==NULL) {
+    return RETOK;
+  }
+  if (x1==NULL || x2==NULL) {
+    return RETFAIL;
+  }
+
+  if (x1->num != x2->num) {
+    return RETFAIL;
+  }
+
+  qsort(x1->ents, x1->num, sizeof(xattr_node), cmp_xattr_node);
+  qsort(x2->ents, x2->num, sizeof(xattr_node), cmp_xattr_node);
+  
+  while (num++ < x1->num)
+  {
+    const char *x1key = NULL;
+    const byte *x1val = NULL;
+    size_t x1vsz = 0;
+    const char *x2key = NULL;
+    const byte *x2val = NULL;
+    size_t x2vsz = 0;
+    
+    x1key = x1->ents[num - 1].key;
+    x1val = x1->ents[num - 1].val;
+    x1vsz = x1->ents[num - 1].vsz;
+
+    x2key = x2->ents[num - 1].key;
+    x2val = x2->ents[num - 1].val;
+    x2vsz = x2->ents[num - 1].vsz;
+
+    if (strcmp(x1key, x2key) ||
+        x1vsz != x2vsz ||
+        memcmp(x1val, x2val, x1vsz))
+      return RETFAIL;
+  }
+  
+  return RETOK;
+}
+
+static int bytecmp(byte *b1, byte *b2, size_t len)
+{
+  return strncmp((char *)b1, (char *)b2, len);
+}
+
 int compare_md_entries(byte* e1,byte* e2,int len)
 {
 
   error(255,"Debug, compare_md_entries %p %p\n",e1,e2);
 
   if(e1!=NULL && e2!=NULL){
-    if(strncmp(e1,e2,len)!=0){
+    if(bytecmp(e1,e2,len)!=0){
       return RETFAIL;
     }else{
       return RETOK;
@@ -135,6 +250,21 @@ int compare_md_entries(byte* e1,byte* e2,int len)
   return RETFAIL;
 }
 
+static int compare_str(const char *s1, const char *s2)
+{
+  if(s1==NULL){
+    if(s2!=NULL){
+      return RETFAIL;
+    }
+  }else if(s2==NULL){
+    return RETFAIL;
+  }else if (strcmp(s1,s2)!=0){
+    return RETFAIL;
+  }
+
+  return RETOK;
+}
+
 
 /*
   We assume
@@ -144,7 +274,7 @@ int compare_md_entries(byte* e1,byte* e2,int len)
   - ignorelist kertoo mit ei saa vertailla
 */
 
-int compare_dbline(db_line* l1,db_line* l2,int ignorelist)
+int compare_dbline(db_line* l1,db_line* l2,DB_ATTR_TYPE ignorelist)
 {
 
 #define easy_compare(a,b) \
@@ -163,20 +293,11 @@ int compare_dbline(db_line* l1,db_line* l2,int ignorelist)
   }
   
   
-  int ret=0;
+  DB_ATTR_TYPE ret=0;
   
   if (!(DB_LINKNAME&ignorelist)) {
-    if(l1->linkname==NULL){
-      if(l2->linkname!=NULL){
+    if(compare_str(l1->linkname, l2->linkname)){
 	ret|=DB_LINKNAME;
-	//return RETFAIL;
-      }
-    }else if(l2->linkname==NULL){
-	ret|=DB_LINKNAME;
-	//return RETFAIL;
-    }else if(strcmp(l1->linkname,l2->linkname)!=0){
-	ret|=DB_LINKNAME;
-	//return RETFAIL;
     }
   }
     
@@ -184,12 +305,10 @@ int compare_dbline(db_line* l1,db_line* l2,int ignorelist)
     if ( (DB_SIZEG&l2->attr) && !(DB_SIZE&l2->attr) ){
       if(l1->size>l2->size){
 	ret|=DB_SIZEG;
-	//return RETFAIL;
       }
     } else {
       if(l1->size!=l2->size){
 	ret|=DB_SIZEG;
-	//return RETFAIL;
       }
     }
   }
@@ -199,7 +318,6 @@ int compare_dbline(db_line* l1,db_line* l2,int ignorelist)
   if (!(DB_PERM&ignorelist)) {
     if(l1->perm!=l2->perm){
       ret|=DB_PERM;
-      //return RETFAIL;
     }
   } else {
     error(0,"Ignoring permissions\n");
@@ -217,17 +335,22 @@ int compare_dbline(db_line* l1,db_line* l2,int ignorelist)
 
   easy_md_compare(DB_MD5,md5,HASH_MD5_LEN);
   
-  error(255,"Debug, %s, %p %p %i %i\n",l1->filename,l1->md5,l2->md5,ret&DB_MD5,ignorelist);
+  error(255,"Debug, %s, %p %p %llx %llx\n",
+        l1->filename,l1->md5,l2->md5,ret&DB_MD5,ignorelist);
   
   easy_md_compare(DB_SHA1,sha1,HASH_SHA1_LEN);
   easy_md_compare(DB_RMD160,rmd160,HASH_RMD160_LEN);
   easy_md_compare(DB_TIGER,tiger,HASH_TIGER_LEN);
   
+  easy_md_compare(DB_SHA256,sha256,HASH_SHA256_LEN);
+  easy_md_compare(DB_SHA512,sha512,HASH_SHA512_LEN);
+  
 #ifdef WITH_MHASH
   easy_md_compare(DB_CRC32,crc32,HASH_CRC32_LEN);
   easy_md_compare(DB_HAVAL,haval,HASH_HAVAL256_LEN);
   easy_md_compare(DB_GOST,gost,HASH_GOST_LEN);
   easy_md_compare(DB_CRC32B,crc32b,HASH_CRC32B_LEN);
+  easy_md_compare(DB_WHIRLPOOL,whirlpool,HASH_WHIRLPOOL_LEN);
 #endif
 
 #ifdef WITH_ACL
@@ -237,10 +360,20 @@ int compare_dbline(db_line* l1,db_line* l2,int ignorelist)
     }
   }
 #endif
+  if (!(DB_XATTRS&ignorelist)) {
+    if(compare_xattrs(l1->xattrs,l2->xattrs)) {
+      ret|=DB_XATTRS;
+    }
+  }
+  if (!(DB_SELINUX&ignorelist)) {
+    if(compare_str(l1->cntx,l2->cntx)) {
+      ret|=DB_SELINUX;
+    }
+  }
   return ret;
 }
 
-void print_lname_changes(char*old,char*new)
+void print_str_changes(char*old,char*new,const char *name)
 {
   int ok = 0;
 
@@ -260,46 +393,120 @@ void print_lname_changes(char*old,char*new)
         ok = 1;
   }
    if(ok)
-     error(5,(char*)entry_format,"Lname",oline,nline);
+     error(2,(char*)entry_format,name,oline,nline);
 
    return;
 }
 
 #ifdef WITH_ACL
-void print_single_acl(acl_type* acl){
-  char* aclt;
-  
+void print_single_acl(acl_type* acl)
+{
   if (acl==NULL) {
-    error(5,"<NULL>");
+    error(2,"<NULL>\n");
   } else {
-    
+#ifdef WITH_POSIX_ACL
+    if (!acl->acl_a)
+      error(2,"A:<NONE>\n           ");
+    else
+      error(2,"A:\n----\n%s----\n           ",acl->acl_a);
+    if (!acl->acl_d)
+      error(2,"D:<NONE>\n");
+    else
+      error(2,"D:\n----\n%s----\n",acl->acl_d);
+#endif
+#ifdef WITH_SUN_ACL
     aclt=acltotext(acl->acl,acl->entries);
     if (aclt==NULL) {
-      error(5,"ERROR");
+      error(2,"ERROR\n");
     } else {
-      error(5,"%s ,",aclt);
+      error(2,"%s ,\n",aclt);
       free(aclt);
     }
+#endif
   }
 }
 
 void print_acl_changes(acl_type* old,acl_type* new) {
   
   if (compare_acl(old,new)==RETFAIL) {
-    error(5,"Acl: old = ");
+    error(2,"ACL: old = ");
     print_single_acl(old);
-    error(5,"\n     new = ");
+    error(2,"     new = ");
     print_single_acl(new);
   }
+}
+#endif
+
+static size_t xstrnspn(const char *s1, size_t len, const char *srch)
+{
+  const char *os1 = s1;
+  
+  while (len-- && strchr(srch, *s1))
+    ++s1;
+
+  return (s1 - os1);
+}
+
+#define PRINTABLE_XATTR_VALS                    \
+    "0123456789"                                \
+    "abcdefghijklmnopqrstuvwxyz"                \
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"                \
+    ".-_:;,[]{}<>()!@#$%^&*|\\/?~"
+
+void print_single_xattrs(xattrs_type* xattrs)
+{
+  if (xattrs==NULL) {
+    error(2,"num=0\n");
+  } else {
+    size_t num = 0;
+    int width = 0;
+    
+    error(2,"num=%d\n", xattrs->num);
+
+    width = log10(xattrs->num); /* make them the same width */
+    
+    while (num++ < xattrs->num)
+    {
+      char *val = NULL;
+      size_t len = 0;
+      
+      val = (char *)xattrs->ents[num - 1].val;
+
+      len = xstrnspn(val, xattrs->ents[num - 1].vsz, PRINTABLE_XATTR_VALS);
+      
+      if ((len ==  xattrs->ents[num - 1].vsz) ||
+          ((len == (xattrs->ents[num - 1].vsz - 1)) && !val[len]))
+        error(2,"  [%.*zd] %s = %s\n", width, num,
+              xattrs->ents[num - 1].key, val);
+      else        
+      {
+        val = encode_base64(xattrs->ents[num - 1].val,
+                            xattrs->ents[num - 1].vsz);
+        error(2,"  [%.*zd] %s <=> %s\n", width, num,
+              xattrs->ents[num - 1].key, val);
+        free(val);
+      }
+      
+    }
+  }
+}
+
+void print_xattrs_changes(xattrs_type* old,xattrs_type* new) {
+  
+  if (compare_xattrs(old,new)==RETFAIL) {
+    error(2,"XAttrs: old = ");
+    print_single_xattrs(old);
+    error(2,"        new = ");
+    print_single_xattrs(new);
+  }
   
 }
-#endif
 
 void print_md_changes(byte*old,byte*new,int len,char* name)
 {
   int ok = 0;
   if(old!=NULL && new!=NULL){
-    if(strncmp(old,new,len)!=0){
+    if(bytecmp(old,new,len)!=0){
       snprintf(oline,part_len,"%s",encode_base64(old,len));
       snprintf(nline,part_len,"%s",encode_base64(new,len));
       ok = 1;
@@ -309,21 +516,21 @@ void print_md_changes(byte*old,byte*new,int len,char* name)
       return;
     }
     if(old==NULL){
-      snprintf(oline,part_len,"NA");
+      snprintf(oline,part_len,"<NONE>");
     } else {
       snprintf(oline,part_len,"%s",encode_base64(old,len));
       ok = 1;
     }
     /* OLD one */
     if(new==NULL){
-      snprintf(nline,part_len,"NA");
+      snprintf(nline,part_len,"<NONE>");
     }else {
       snprintf(nline,part_len,"%s",encode_base64(new,len));
       ok = 1;
     }
   }
   if(ok)
-    error(5,(char*)entry_format,name,oline,nline);
+    error(2,(char*)entry_format,name,oline,nline);
   
   return;
 }
@@ -335,7 +542,7 @@ int is_time_null(struct tm *ot)
             && ot->tm_hour == 1 &&  ot->tm_min == 0 && ot->tm_sec == 0);
 }
 
-void print_time_changes(const char* name, time_t old_time, time_t new_time)
+void print_time_changes(const char* name, time_t old_time, time_t new_time,int justnew)
 {
   struct tm otm;
   struct tm *ot = &otm;
@@ -348,88 +555,113 @@ void print_time_changes(const char* name, time_t old_time, time_t new_time)
   ot->tm_min = tmp->tm_min; ot->tm_sec = tmp->tm_sec;
   
   nt = localtime(&(new_time));
-  
-  if( is_time_null(ot) )
-    snprintf(oline,part_len,"NA");
-  else
-    snprintf(oline,part_len,
-	     "%0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u",
-	     ot->tm_year+1900, ot->tm_mon+1, ot->tm_mday,
-	     ot->tm_hour, ot->tm_min, ot->tm_sec);
-  if( is_time_null(nt) )
-    snprintf(nline,part_len,"NA");
-  else
+  if (!justnew) {
+    if( is_time_null(ot) ) {
+      snprintf(oline,part_len,"<NONE>");
+    } else {
+      snprintf(oline,part_len,
+	       "%.4u-%.2u-%.2u %.2u:%.2u:%.2u",
+	       ot->tm_year+1900, ot->tm_mon+1, ot->tm_mday,
+	       ot->tm_hour, ot->tm_min, ot->tm_sec);
+    }
+  }
+  if( is_time_null(nt) ) {
+    snprintf(nline,part_len,"<NONE>");
+  } else {
     snprintf(nline,part_len,
-	     "%0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u",
+	     "%.4u-%.2u-%.2u %.2u:%.2u:%.2u",
 	     nt->tm_year+1900, nt->tm_mon+1, nt->tm_mday,
 	     nt->tm_hour, nt->tm_min, nt->tm_sec);
-  error(5,(char*)entry_format,name,oline,nline); 
+  }
+  if (justnew) {
+    error(2,(char*)entry_format_justnew,name,' ',nline);
+  } else {
+    error(2,(char*)entry_format,name,oline,nline); 
+  }
 }
 
-void print_int_changes(
-        const char* name,
-        int old,
-        int new
-        )
+void print_int_changes(const char* name, int old, int new, int justnew)
 {
-  snprintf(oline,part_len,"%i",old);
+  if (!justnew) {
+    snprintf(oline,part_len,"%i",old);
+  }
   snprintf(nline,part_len,"%i",new);
-  error(5,(char*)entry_format,name,oline,nline); 
+  if (justnew) {
+    error(2,(char*)entry_format_justnew,name,' ',nline);
+  } else {
+    error(2,(char*)entry_format,name,oline,nline);
+  }
 }
-void print_ulong_changes(
-        const char* name,
-        unsigned long old,
-        unsigned long new
-        )
+void print_long_changes(const char* name, AIDE_OFF_TYPE old, AIDE_OFF_TYPE new, int justnew)
 {
-  snprintf(oline,part_len,"%lu",old);
+#if AIDE_OFF_TYPE == off64_t
+  if (!justnew) {
+    snprintf(oline,part_len,"%llu",old);
+  }
+  snprintf(nline,part_len,"%llu",new);
+#else
+  if (!justnew) {
+    snprintf(oline,part_len,"%lu",old);
+  }
   snprintf(nline,part_len,"%lu",new);
-  error(5,(char*)entry_format,name,oline,nline);  
+#endif
+  if (justnew) {
+    error(2,(char*)entry_format,name,nline);
+  } else {
+    error(2,(char*)entry_format,name,oline,nline);
+  }
 }
 
-void print_string_changes(
-        const char* name,
-        const char* old,
-        const char* new
-        )
+void print_string_changes(const char* name, const char* old, const char* new, int justnew)
 {
-  snprintf(oline,part_len,"%s",old);
+  if (!justnew) {
+    snprintf(oline,part_len,"%s",old);
+  }
   snprintf(nline,part_len,"%s",new);
-  error(5,(char*)entry_format,name,oline,nline); 
+  if (justnew) {
+    error(2,(char*)entry_format,name,nline); 
+  } else {
+    error(2,(char*)entry_format,name,oline,nline); 
+  }
 }
 
 
-void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
+void print_dbline_changes(db_line* old,db_line* new,
+                          DB_ATTR_TYPE ignorelist,DB_ATTR_TYPE forced_attrs)
 {
   char* tmp=NULL;
   char* tmp2=NULL;
   
-
+  /*
+    Force just entries, that exists.
+  */
+  forced_attrs&=new->attr;
+  
   if(S_ISDIR(new->perm_o)){
-    error(5,"\nDirectory: %s\n",old->filename);
+    error(2,"\nDirectory: %s\n",old->filename);
   }else {
-    error(5,"\nFile: %s\n",old->filename);
+    error(2,"\nFile: %s\n",old->filename);
   }
   
   if(!(DB_LINKNAME&ignorelist)){
-    print_lname_changes(old->linkname,new->linkname);
+    print_str_changes(old->linkname,new->linkname, "Lname");
   }
   if (!(DB_SIZE&ignorelist)) {
-    if(old->size!=new->size){
-      print_ulong_changes("Size", old->size,new->size);
+    if(old->size!=new->size||(DB_SIZE&forced_attrs)){
+      print_long_changes("Size", old->size,new->size,old->size==new->size);
     }
   }
 
   if (!(DB_BCOUNT&ignorelist)) {
-    if(old->bcount!=new->bcount){
-      print_int_changes("Bcount", old->bcount,new->bcount);
+    if(old->bcount!=new->bcount ||(DB_BCOUNT&forced_attrs) ){
+      print_int_changes("Bcount", old->bcount,new->bcount,old->bcount==new->bcount);
     }
   }
   if (!(DB_PERM&ignorelist)) {
-    if(old->perm!=new->perm){
+    if(old->perm!=new->perm || DB_PERM&forced_attrs){
       tmp=perm_to_char(old->perm);
       tmp2=perm_to_char(new->perm);
-      print_string_changes("Permissions", tmp,tmp2);
+      print_string_changes("Permissions", tmp,tmp2,old->perm==new->perm);
       free(tmp);
       free(tmp2);
       tmp=NULL;
@@ -438,43 +670,43 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
   }
   
   if (!(DB_UID&ignorelist)) {
-    if(old->uid!=new->uid){
-      print_int_changes("Uid", old->uid,new->uid);
+    if(old->uid!=new->uid||DB_UID&forced_attrs){
+      print_int_changes("Uid", old->uid,new->uid,old->uid==new->uid);
     }
   }
   
   if (!(DB_GID&ignorelist)) {
-    if(old->gid!=new->gid){
-      print_int_changes("Gid", old->gid,new->gid);
+    if(old->gid!=new->gid||DB_GID&forced_attrs){
+      print_int_changes("Gid", old->gid,new->gid,old->gid==new->gid);
     }
   }
   
   if (!(DB_ATIME&ignorelist)) {
-    if(old->atime!=new->atime){
-      print_time_changes("Atime", old->atime, new->atime);
+    if(old->atime!=new->atime||DB_ATIME&forced_attrs){
+      print_time_changes("Atime", old->atime, new->atime,old->atime==new->atime);
     }
   }
   
   if (!(DB_MTIME&ignorelist)) {
-    if(old->mtime!=new->mtime){
-      print_time_changes("Mtime", old->mtime, new->mtime);
+    if(old->mtime!=new->mtime||DB_MTIME&forced_attrs){
+      print_time_changes("Mtime", old->mtime, new->mtime,old->mtime==new->mtime);
     }
   }
   
   if (!(DB_CTIME&ignorelist)) {
-    if(old->ctime!=new->ctime){
-      print_time_changes("Ctime", old->ctime, new->ctime);
+    if(old->ctime!=new->ctime||DB_CTIME&forced_attrs){
+      print_time_changes("Ctime", old->ctime, new->ctime,old->ctime==new->ctime);
     }
   }
 
   if (!(DB_INODE&ignorelist)) {
-    if(old->inode!=new->inode){
-      print_int_changes("Inode", old->inode,new->inode);
+    if(old->inode!=new->inode||DB_INODE&forced_attrs){
+      print_int_changes("Inode", old->inode,new->inode,old->inode==new->inode);
     }
   }
   if (!(DB_LNKCOUNT&ignorelist)) {
-    if(old->nlink!=new->nlink){
-      print_int_changes("Linkcount", old->nlink,new->nlink);
+    if(old->nlink!=new->nlink||DB_LNKCOUNT&forced_attrs){
+      print_int_changes("Linkcount", old->nlink,new->nlink,old->nlink==new->nlink);
     }
   }
 
@@ -502,6 +734,18 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
 		     "TIGER");
   }
   
+  if (!(DB_SHA256&ignorelist)) {
+      print_md_changes(old->sha256,new->sha256,
+		       HASH_SHA256_LEN,
+		       "SHA256");
+  }
+
+  if (!(DB_SHA512&ignorelist)) {
+      print_md_changes(old->sha512,new->sha512,
+		       HASH_SHA512_LEN,
+		       "SHA512");
+  }
+
 #ifdef WITH_MHASH
   if (!(DB_CRC32&ignorelist)) {
     print_md_changes(old->crc32,new->crc32,
@@ -526,6 +770,12 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
 		     HASH_CRC32B_LEN,
 		     "CRC32B");
   }
+
+  if (!(DB_WHIRLPOOL&ignorelist)) {
+      print_md_changes(old->whirlpool,new->whirlpool,
+		       HASH_WHIRLPOOL_LEN,
+		       "WHIRLPOOL");
+  }
 #endif                   
 
 #ifdef WITH_ACL
@@ -533,6 +783,12 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
     print_acl_changes(old->acl,new->acl);
   }
 #endif
+  if (!(DB_XATTRS&ignorelist)) {
+    print_xattrs_changes(old->xattrs,new->xattrs);
+  }
+  if (!(DB_SELINUX&ignorelist)) {
+    print_str_changes(old->cntx,new->cntx, "SELinux");
+  }
   
   return;
 }
@@ -606,19 +862,19 @@ void print_report_header(int nfil,int nadd,int nrem,int nchg)
   if(conf->action&DO_DIFF)
     error(0,_("AIDE found differences between the two databases!!\n"));
   if(conf->config_version)
-    error(5,_("Config version used: %s\n"),conf->config_version);
+    error(2,_("Config version used: %s\n"),conf->config_version);
 
-  error(5,_("Start timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n"),
+  error(2,_("Start timestamp: %.4u-%.2u-%.2u %.2u:%.2u:%.2u\n"),
 	st->tm_year+1900, st->tm_mon+1, st->tm_mday,
 	st->tm_hour, st->tm_min, st->tm_sec);
-  error(0,_("Summary:\nTotal number of files=%i,added files=%i"
-	    ",removed files=%i,changed files=%i\n\n"),nfil,nadd,nrem,nchg);
+  error(0,_("\nSummary:\n  Total number of files:\t%i\n  Added files:\t\t\t%i\n"
+	    "  Removed files:\t\t%i\n  Changed files:\t\t%i\n\n"),nfil,nadd,nrem,nchg);
   
 }
 
 void print_report_footer(struct tm* st)
 {
-    error(20,_("\nEnd timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n"),
+    error(2,_("\nEnd timestamp: %.4u-%.2u-%.2u %.2u:%.2u:%.2u\n"),
 	  st->tm_year+1900, st->tm_mon+1, st->tm_mday,
 	  st->tm_hour, st->tm_min, st->tm_sec);
 }
@@ -637,10 +893,11 @@ void compare_db(list* new,db_config* conf)
   long nadd=0;
   long nfil=0;
   long filesindir=0;
-  int tempignore=0;
+  DB_ATTR_TYPE tempignore=0;
   int initdbwarningprinted=0;
 
-  int ignorelist;
+  DB_ATTR_TYPE ignorelist;
+  DB_ATTR_TYPE forced_attrs;
 
   error(200,_("compare_db()\n"));
 
@@ -658,10 +915,12 @@ void compare_db(list* new,db_config* conf)
   
   /* We have a way to ignore some changes... */ 
   
-  ignorelist=get_groupval("ignore_list");
+  ignorelist=get_ignorelist();
   
-  if (ignorelist==-1) {
-    ignorelist=0;
+  forced_attrs=get_report_attributes();
+
+  if (forced_attrs==DB_ATTR_UNDEF) {
+    forced_attrs=0;
   }
 
   for(old=db_readline(DB_OLD);old;old=db_readline(DB_OLD)){
@@ -676,7 +935,7 @@ void compare_db(list* new,db_config* conf)
 	  check_list_for_match(conf->equrxlst,old->filename,&tempignore)) &&
 	 !check_list_for_match(conf->negrxlst,old->filename,&tempignore)){
 	if(!(conf->action&DO_INIT)){
-	  error(5,_("WARNING: Old db contains a file that shouldn\'t be there, run --init or --update\n"));
+	  error(2,_("WARNING: Old db contains a file that shouldn\'t be there, run --init or --update\n"));
 	}
 	initdbwarningprinted=1;
       }
@@ -685,8 +944,9 @@ void compare_db(list* new,db_config* conf)
     }else {
       int localignorelist=old->attr ^ ((db_line*)r->data)->attr;
       
-      if (localignorelist!=0) {
-	error(5,"File %s in databases has different attributes, %i,%i\n",old->filename,old->attr,((db_line*)r->data)->attr);
+      if ((localignorelist&(~(DB_NEWFILE|DB_RMFILE)))!=0) {
+	error(2,"File %s in databases has different attributes, %llx,%llx\n",
+              old->filename,old->attr,((db_line*)r->data)->attr);
       }
       
       localignorelist|=ignorelist;
@@ -700,14 +960,9 @@ void compare_db(list* new,db_config* conf)
       }else {
 	/* This line was ok */
 	/*
-	  Cannot free, chech why.
+	  Cannot free, check why.
 	  It's because db_disk needs it for going back
 	  to it's parent.
-	  
-	  free_db_line(old);
-	  free(old);
-	  free_db_line((db_line*)r->data);
-	  free((db_line*)r->data);
 	*/
 	
 	l=list_delete_item(r);
@@ -727,24 +982,20 @@ void compare_db(list* new,db_config* conf)
     nadd++;
   }
 
-
   if(nadd!=0||nrem!=0||nchg!=0){
     print_report_header(nfil,nadd,nrem,nchg);
 
     if(nadd!=0){
-      error(5,_("Added files:\n"));
+      error(2,_("\n---------------------------------------------------\n"));
+      error(2,_("Added files:\n"));
+      error(2,_("---------------------------------------------------\n\n"));
       for(r=added;r;r=r->next){
-	error(5,"added:%s\n",((db_line*)r->data)->filename);
+	error(2,"added: %s\n",((db_line*)r->data)->filename);
 	if(conf->verbose_level<20){
 	  if(S_ISDIR(((db_line*)r->data)->perm)){
-	    /*	    
-		    free_db_line((db_line*)r->data);
-		    free(r->data);
-		    r=list_delete_item(r);
-	    */
 	    eat_files_indir(r->next,((db_line*)r->data)->filename,&filesindir);
 	    if(filesindir>0){
-	      error(5,
+	      error(2,
 		    _("added: THERE WERE ALSO %li "
 		    "FILES ADDED UNDER THIS DIRECTORY\n")
 		    ,filesindir);
@@ -756,26 +1007,32 @@ void compare_db(list* new,db_config* conf)
     
 
     if(nrem!=0){
-      error(5,_("Removed files:\n"));
+      error(2,_("\n---------------------------------------------------\n"));
+      error(2,_("Removed files:\n"));
+      error(2,_("---------------------------------------------------\n\n"));
       for(r=removed;r;r=r->next){
-	error(5,"removed:%s\n",((db_line*)r->data)->filename);
+	error(2,"removed: %s\n",((db_line*)r->data)->filename);
       }
     }
 
     if(nchg!=0){
-      error(5,_("Changed files:\n"));
+      error(2,_("\n---------------------------------------------------\n"));
+      error(2,_("Changed files:\n"));
+      error(2,_("---------------------------------------------------\n\n"));
       for(r=changedold;r;r=r->next){
-	error(5,"changed:%s\n",((db_line*)r->data)->filename);
+	error(2,"changed: %s\n",((db_line*)r->data)->filename);
       }
     }
 
     if((conf->verbose_level>=5)&&(nchg!=0)){
-      error(5,_("Detailed information about changes:\n"));
+      error(2,_("\n---------------------------------------------------\n"));
+      error(2,_("Detailed information about changes:\n"));
+      error(2,_("---------------------------------------------------\n\n"));
       for(r=changedold,l=changednew;r;r=r->next,l=l->next){
-	int localignorelist=((db_line*)l->data)->attr^((db_line*)r->data)->attr;
+	DB_ATTR_TYPE localignorelist=((db_line*)l->data)->attr^((db_line*)r->data)->attr;
 	localignorelist|=ignorelist;
 	print_dbline_changes((db_line*)r->data,
-			     (db_line*)l->data,localignorelist);
+			     (db_line*)l->data,localignorelist,forced_attrs);
       }
     }
     conf->end_time=time(&(conf->end_time));
@@ -783,17 +1040,40 @@ void compare_db(list* new,db_config* conf)
   }
 }
 
-long report_tree(seltree* node,int stage, int* stat)
+  /* Something changed, send audit anomaly message */
+void send_audit_report(long nadd, long nrem, long nchg)
+{
+#ifdef WITH_AUDIT
+  if(nadd!=0||nrem!=0||nchg!=0){
+    int fd=audit_open();
+    if (fd>=0){
+       char msg[64];
+
+       snprintf(msg, sizeof(msg), "added=%ld removed=%ld changed=%ld", 
+                nadd, nrem, nchg);
+
+       if (audit_log_user_message(fd, AUDIT_ANOM_RBAC_INTEGRITY_FAIL,
+                                  msg, NULL, NULL, NULL, 0)<=0)
+#ifdef HAVE_SYSLOG
+          syslog(LOG_ERR, "Failed sending audit message:%s", msg);
+#else
+          ;
+#endif
+       close(fd);
+    }
+  }
+#endif /* WITH_AUDIT */
+}
+
+long report_tree(seltree* node,int stage, long* stat)
 {
   list* r=NULL;
   int ignorelist=0;
+  int forced_attrs=0;
   int top=0;
 
-  ignorelist=get_groupval("ignore_list");
-  
-  if (ignorelist==-1) {
-    ignorelist=0;
-  }
+  ignorelist=get_ignorelist();
+  forced_attrs=get_report_attributes();
   
   if(stat[0]){
     stat[0]=0;
@@ -816,50 +1096,77 @@ long report_tree(seltree* node,int stage, int* stat)
 	/* Node was added to twice and discovered to be not changed*/
       }else if(!(node->checked&DB_OLD)&&(node->checked&DB_NEW)){
 	/* File is in new db but not old. (ADDED) */
-	stat[2]++;
-	node->checked|=NODE_ADDED;
+	/* unless it was moved in */
+	if (!((node->checked&NODE_ALLOW_NEW)||(node->checked&NODE_MOVED_IN))) {
+	  stat[2]++;
+	  node->checked|=NODE_ADDED;
+	}
       }else if((node->checked&DB_OLD)&&!(node->checked&DB_NEW)){
 	/* File is in old db but not new. (REMOVED) */
-	stat[3]++;
-	node->checked|=NODE_REMOVED;
+	/* unless it was moved out */
+	if (!((node->checked&NODE_ALLOW_RM)||(node->checked&NODE_MOVED_OUT))) {
+	  stat[3]++;
+	  node->checked|=NODE_REMOVED;
+	}
       }else if((node->checked&DB_OLD)&&(node->checked&DB_NEW)&&
 	       (node->old_data!=NULL)&&(node->new_data!=NULL)){
 	/* File is in both db's and the data is still there. (CHANGED) */
-	stat[4]++;
-	node->checked|=NODE_CHANGED;
+	if(!(node->checked&(NODE_MOVED_IN|NODE_MOVED_OUT))){
+	  stat[4]++;
+	  node->checked|=NODE_CHANGED;
+	}else if (!((node->checked&NODE_ALLOW_NEW)||(node->checked&NODE_MOVED_IN))) {
+	  stat[2]++;
+	  node->checked|=NODE_ADDED;
+	}else if (!((node->checked&NODE_ALLOW_RM)||(node->checked&NODE_MOVED_OUT))) {
+	  stat[3]++;
+	  node->checked|=NODE_REMOVED;
+	}
       }
     }
   }
 
   if((stage==1)&&stat[2]){
-    if(top)
-      error(5,_("Added files:\n"));
+    if(top){
+      error(2,_("\n---------------------------------------------------\n"));
+      error(2,_("Added files:\n"));
+      error(2,_("---------------------------------------------------\n\n"));
+    }
     if(node->checked&NODE_ADDED){
-      error(5,_("added:%s\n"),node->new_data->filename);
+      error(2,_("added: %s\n"),node->new_data->filename);
     }
   }
 
   if((stage==2)&&stat[3]){
-    if(top)
-      error(5,_("Removed files:\n"));
+    if(top){
+      error(2,_("\n---------------------------------------------------\n"));
+      error(2,_("Removed files:\n"));
+      error(2,_("---------------------------------------------------\n\n"));
+    }
     if(node->checked&NODE_REMOVED){
-      error(5,_("removed:%s\n"),node->old_data->filename);
+      error(2,_("removed: %s\n"),node->old_data->filename);
     }
   }
 
   if((stage==3)&&stat[4]){
-    if(top)
-      error(5,_("Changed files:\n"));
+    if(top){
+      error(2,_("\n---------------------------------------------------\n"));
+      error(2,_("Changed files:\n"));
+      error(2,_("---------------------------------------------------\n\n"));
+    }
     if(node->checked&NODE_CHANGED){
-      error(5,_("changed:%s\n"),node->new_data->filename);
+      error(2,_("changed: %s\n"),node->new_data->filename);
     }
   }
 
   if((stage==4)&&(conf->verbose_level>=5)&&stat[4]){
-    if(top)
-      error(5,_("Detailed information about changes:\n"));
+    if(top){
+      error(2,_("\n--------------------------------------------------\n"));
+      error(2,_("Detailed information about changes:\n"));
+      error(2,_("---------------------------------------------------\n\n"));
+    }
     if(node->checked&NODE_CHANGED){
-      print_dbline_changes(node->old_data,node->new_data,ignorelist);
+      int localignorelist=(node->old_data->attr ^ node->new_data->attr)|ignorelist;
+      print_dbline_changes(node->old_data,node->new_data,localignorelist,forced_attrs);
     }
   }
 
@@ -869,6 +1176,7 @@ long report_tree(seltree* node,int stage, int* stat)
   }
 
   if(top&&(stage==0)&&((stat[2]+stat[3]+stat[4])>0)){
+    send_audit_report(stat[2],stat[3],stat[4]);
     print_report_header(stat[1],stat[2],stat[3],stat[4]);
   }
   
@@ -877,3 +1185,5 @@ long report_tree(seltree* node,int stage, int* stat)
 
 const char* aide_key_9=CONFHMACKEY_09;
 const char* db_key_9=DBHMACKEY_09;
+
+// vi: ts=8 sw=8
