@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/db.c,v 1.2 2003/11/04 15:13:06 rvdb Exp $
+ * $Header: /cvsroot/aide/aide/src/db.c,v 1.16 2006/11/25 10:07:40 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
  
-#include "../config.h"
+#include "aide.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -34,25 +34,101 @@
 #include "report.h"
 #include "be.h"
 
-/*
-#include <gcrypt.h>
-*/
 #ifdef WITH_MHASH
 #include <mhash.h>
 #endif
 
 #include "base64.h"
 #include "util.h"
-#include "aide.h"
 /*for locale support*/
 #include "locale-aide.h"
 /*for locale support*/
 
 db_line* db_char2line(char** ss,int db);
 long readint(char* s,char* err);
+AIDE_OFF_TYPE readlong(char* s,char* err);
 long readoct(char* s,char* err);
-
 time_t base64totime_t(char*);
+
+const char* db_names[db_unknown+1] = {
+   "name",
+   "lname",
+   "perm",
+   "uid",
+   "gid",
+   "size",
+   "atime",
+   "ctime",
+   "mtime",
+   "inode",
+   "bcount",
+   "lcount",
+   "md5",
+   "sha1",
+   "rmd160",
+   "tiger",
+   "crc32",
+   "haval",
+   "gost",
+   "crc32b",
+   "attr",
+   "acl",
+   "bsize",
+   "rdev",
+   "dev",
+   "checkmask",
+   "allownewfiles",
+   "allowrmfiles",
+   "sha256",
+   "sha512",
+   "whirlpool",
+   "selinux",
+   "xattrs",
+   "unknown"} ;
+
+const int db_value[db_unknown+1] = {
+   db_filename,         /* "name",   */
+   db_linkname,         /* "lname",   */
+   db_perm,             /* "perm",    */
+   db_uid,              /* "uid",     */
+   db_gid,              /* "gid",     */
+   db_size,             /* "size",    */
+   db_atime,            /* "atime",   */
+   db_ctime,            /* "ctime",   */
+   db_mtime,            /* "mtime",   */
+   db_inode,            /* "inode",   */
+   db_bcount,           /* "bcount",  */
+   db_lnkcount,         /* "lcount",  */
+   db_md5,              /* "md5",     */
+   db_sha1,             /* "sha1",    */
+   db_rmd160,           /* "rmd160",  */
+   db_tiger,            /* "tiger",   */
+   db_crc32,            /* "crc32",   */
+   db_haval,            /* "haval",   */
+   db_gost,             /* "gost",    */
+   db_crc32b,           /* "crc32b",  */
+   db_attr,             /* attributes */
+   db_acl,              /* "acl"      */
+   db_bsize,            /* "bsize"    */
+   db_rdev,             /* "rdev"     */
+   db_dev,              /* "dev"      */
+   db_checkmask,        /* "checkmask" */
+   db_allownewfile,     /* "allownewfile" */
+   db_allowrmfile,      /* "allowrmfile" */
+   db_sha256,           /* "sha256",  */
+   db_sha512,           /* "sha512",  */
+   db_whirlpool,        /* "whirlpool", */
+   db_selinux,          /* "selinux", */
+   db_xattrs,           /* "xattrs",  */
+   db_unknown };        /* "unknown"  */
+
+const char* db_namealias[db_alias_size] = {
+  "count" } ;
+
+const int db_aliasvalue[db_alias_size] = {
+  db_lnkcount } ;       /* "count",  */
+
+
 int db_init(int db)
 {
   void* rv=NULL;
@@ -148,6 +224,11 @@ db_line* db_readline(int db){
   }
 
   switch (db_url->type) {
+#ifdef WITH_CURL
+  case url_http:
+  case url_https:
+  case url_ftp:
+#endif /* WITH CURL */
   case url_stdin:
   case url_fd:
   case url_file: {
@@ -172,10 +253,12 @@ db_line* db_readline(int db){
     
     break;
   }
+
+
 #ifdef WITH_PSQL
   case url_sql: {
     error(255,"db_sql readline...");
-    s=db_readline_sql(db);
+    s=db_readline_sql(db, conf);
     
     break;
   }
@@ -190,14 +273,49 @@ db_line* db_readline(int db){
   
 }
 
-byte* base64tobyte(char* src,int len)
+byte* base64tobyte(char* src,int len,size_t *ret_len)
 {
   if(strcmp(src,"0")!=0){
-    return decode_base64(src,len);
+    return decode_base64(src,len,ret_len);
   }
   return NULL;
 }
 
+static char *db_readchar(char *s)
+{
+  if (s == NULL)
+    return (NULL);
+  
+  if (s[0] == '0')
+  {
+    if (s[1] == '\0')
+      return (NULL);
+    
+    if (s[1] == '-')
+      return (strdup(""));
+
+    if (s[1] == '0')
+    {
+      memmove(s, s+1, strlen(s+1)+1);
+      // Hope this removes core
+      // dumping in some environments. Has something to do with
+      // memory (de)allocation.
+    }
+  }
+
+  decode_string(s);
+
+  return strdup(s);
+}
+
+#define WARN_ONCE(x) case db_ ## x : {                                  \
+      static int warn_once_ ## x = 0;                                   \
+      if (! warn_once_ ## x )                                           \
+        error(0,_("Hash %s uses MHASH, which is not enabled.\n"),       \
+              #x );                                                     \
+      warn_once_ ## x = 1;                                              \
+    } break
+
 db_line* db_char2line(char** ss,int db){
 
   int i;
@@ -229,15 +347,15 @@ db_line* db_char2line(char** ss,int db){
   line->sha1=NULL;
   line->rmd160=NULL;
   line->tiger=NULL;
-#ifdef WITH_MHASH
-  line->crc32=NULL;
+
+  line->crc32=NULL; /* MHASH stuff.. */
   line->crc32b=NULL;
   line->haval=NULL;
   line->gost=NULL;
-#endif
-#ifdef WITH_ACL
-  line->acl=0;
-#endif
+  line->whirlpool=NULL;
+  
+  line->sha256=NULL;
+  line->sha512=NULL;
   line->perm=0;
   line->uid=0;
   line->gid=0;
@@ -250,6 +368,9 @@ db_line* db_char2line(char** ss,int db){
   line->size=0;
   line->filename=NULL;
   line->linkname=NULL;
+  line->acl=NULL;
+  line->xattrs=NULL;
+  line->cntx=NULL;
   
   line->attr=conf->attr; /* attributes from @@dbspec */
 
@@ -266,28 +387,7 @@ db_line* db_char2line(char** ss,int db){
       break;
     }
     case db_linkname : {
-      char *s = ss[(*db_order)[i]];
-      if(ss[(*db_order)[i]]!=NULL){
-	if(ss[(*db_order)[i]][0]=='0'){
-	  if(ss[(*db_order)[i]][1]=='\0'){
-	    line->linkname=NULL;
-	    break;
-	  }else if(ss[(*db_order)[i]][1]=='-'){
-	    line->linkname=strdup("");
-	    break;
-	  }else if(ss[(*db_order)[i]][1]=='0'){
-	    memmove(s,s+1,strlen(s+1)+1); 
-	    // Hope this removes core
-	    // dumping in some environments. Has something to do with
-	    // memory (de)allocation.
-	  }
-	}
-	decode_string(s);
-	line->linkname=strdup(s);
-      } else {
-	error(0,_("db_char2line():Error while reading database\n"));
-	abort();
-      }
+      line->linkname = db_readchar(ss[(*db_order)[i]]);
       break;
     }
     case db_mtime : {
@@ -320,52 +420,71 @@ db_line* db_char2line(char** ss,int db){
       break;
     }
     case db_size : {
-      line->size=readint(ss[(*db_order)[i]],"size");
+      line->size=readlong(ss[(*db_order)[i]],"size");
       break;
     }
     case db_md5 : {
       line->md5=base64tobyte(ss[(*db_order)[i]],
-			     strlen(ss[(*db_order)[i]]));
+			     strlen(ss[(*db_order)[i]]), NULL);
       break;
     }
     case db_sha1 : {
       line->sha1=base64tobyte(ss[(*db_order)[i]],
-			      strlen(ss[(*db_order)[i]]));
+			      strlen(ss[(*db_order)[i]]), NULL);
       break;
     }
     case db_rmd160 : {
       line->rmd160=base64tobyte(ss[(*db_order)[i]],
-				strlen(ss[(*db_order)[i]]));
+				strlen(ss[(*db_order)[i]]), NULL);
       break;
     }
     case db_tiger : {
       line->tiger=base64tobyte(ss[(*db_order)[i]],
-			       strlen(ss[(*db_order)[i]]));
+			       strlen(ss[(*db_order)[i]]), NULL);
       break;
     }
-#ifdef WITH_MHASH
     case db_crc32 : {
       line->crc32=base64tobyte(ss[(*db_order)[i]],
-			       strlen(ss[(*db_order)[i]]));
-      break;
-    }
-    case db_gost : {
-      line->gost=base64tobyte(ss[(*db_order)[i]],
-			       strlen(ss[(*db_order)[i]]));
+			       strlen(ss[(*db_order)[i]]), NULL);
       break;
     }
     case db_haval : {
       line->haval=base64tobyte(ss[(*db_order)[i]],
-			       strlen(ss[(*db_order)[i]]));
+			       strlen(ss[(*db_order)[i]]), NULL);
+      break;
+    }
+#ifdef WITH_MHASH
+    case db_gost : {
+      line->gost=base64tobyte(ss[(*db_order)[i]],
+			       strlen(ss[(*db_order)[i]]), NULL);
       break;
     }
     case db_crc32b : {
       line->crc32b=base64tobyte(ss[(*db_order)[i]],
-			       strlen(ss[(*db_order)[i]]));
+			       strlen(ss[(*db_order)[i]]), NULL);
       break;
     }
+    case db_whirlpool : {
+      line->whirlpool=base64tobyte(ss[(*db_order)[i]],
+                                   strlen(ss[(*db_order)[i]]), NULL);
+      break;
+    }
+#else
+      WARN_ONCE(gost);
+      WARN_ONCE(crc32b);
+      WARN_ONCE(whirlpool);
 #endif
-#ifdef WITH_ACL
+    case db_sha256 : {
+      line->sha256=base64tobyte(ss[(*db_order)[i]],
+                                strlen(ss[(*db_order)[i]]), NULL);
+      break;
+    }
+    case db_sha512 : {
+      line->sha512=base64tobyte(ss[(*db_order)[i]],
+                                strlen(ss[(*db_order)[i]]), NULL);
+      break;
+    }
+#ifdef WITH_SUN_ACL
     case db_acl : {
       char* endp,*pos;
       int entries,lc;
@@ -393,6 +512,70 @@ db_line* db_char2line(char** ss,int db){
       break;
     }
 #endif
+#ifdef WITH_POSIX_ACL
+    case db_acl : {
+      char *tval = NULL;
+      
+      tval = strtok(ss[(*db_order)[i]], ",");
+
+      line->acl = NULL;
+
+      if (tval[0] == '0')
+        line->acl = NULL;
+      else if (!strcmp(tval, "POSIX"))
+      {
+        line->acl = malloc(sizeof(acl_type));        
+        line->acl->acl_a = NULL;
+        line->acl->acl_d = NULL;
+        
+        tval = strtok(NULL, ",");
+        line->acl->acl_a = (char *)base64tobyte(tval, strlen(tval), NULL);
+        tval = strtok(NULL, ",");
+        line->acl->acl_d = (char *)base64tobyte(tval, strlen(tval), NULL);
+      }
+      /* else, it's broken... */
+      break;
+    }
+#endif
+      case db_xattrs : {
+        size_t num = 0;
+        char *tval = NULL;
+        
+        tval = strtok(ss[(*db_order)[i]], ",");
+        num = readlong(tval, "xattrs");
+        if (num)
+        {
+          line->xattrs = malloc(sizeof(xattrs_type));
+          line->xattrs->ents = calloc(sizeof(xattr_node), num);
+          line->xattrs->sz  = num;
+          line->xattrs->num = num;
+          num = 0;
+          while (num < line->xattrs->num)
+          {
+            byte  *val = NULL;
+            size_t vsz = 0;
+            
+            tval = strtok(NULL, ",");
+            line->xattrs->ents[num].key = db_readchar(strdup(tval));
+            tval = strtok(NULL, ",");
+            val = base64tobyte(tval, strlen(tval), &vsz);
+            line->xattrs->ents[num].val = val;
+            line->xattrs->ents[num].vsz = vsz;
+
+            ++num;
+          }
+        }
+        break;
+      }
+
+      case db_selinux : {
+        byte  *val = NULL;
+        
+        val = base64tobyte(ss[(*db_order)[i]], strlen(ss[(*db_order)[i]]),NULL);
+        line->cntx = (char *)val;
+        break;
+      }
+      
     case db_perm : {
       line->perm=readoct(ss[(*db_order)[i]],"permissions");
       break;
@@ -404,7 +587,7 @@ db_line* db_char2line(char** ss,int db){
     }
 
     case db_attr : {
-      line->attr=readint(ss[(*db_order)[i]],"attr");
+      line->attr=readlong(ss[(*db_order)[i]],"attr");
       break;
     }
     
@@ -427,7 +610,7 @@ db_line* db_char2line(char** ss,int db){
 
 time_t base64totime_t(char* s){
   
-  byte* b=decode_base64(s,strlen(s));
+  byte* b=decode_base64(s,strlen(s),NULL);
   char* endp;
   
   if (b==NULL||strcmp(s,"0")==0) {
@@ -437,7 +620,7 @@ time_t base64totime_t(char* s){
     
     return 0;
   } else {
-    time_t t = strtol(b,&endp,10);
+    time_t t = strtol((char *)b,&endp,10);
     
     if (endp[0]!='\0') {
       error(0,"Error converting base64\n");
@@ -461,6 +644,16 @@ long readint(char* s,char* err){
   return i;
 }
 
+AIDE_OFF_TYPE readlong(char* s,char* err){
+  AIDE_OFF_TYPE i;
+  char* e;
+  i=AIDE_STRTOLL_FUNC(s,&e,10);
+  if (e[0]!='\0') {
+    error(0,_("Could not read %s from database"),err);
+  }
+  return i;
+}
+
 long readoct(char* s,char* err){
   long i;
   char* e;
@@ -490,6 +683,15 @@ int db_writespec(db_config* conf)
     }
     break;
   }
+#ifdef WITH_CURL
+  case url_http:
+  case url_https:
+  case url_ftp:
+    {
+      
+      break;
+    }
+#endif /* WITH CURL */
 #ifdef WITH_PSQL
   case url_sql: {
     if(conf->db_out!=NULL){
@@ -513,6 +715,11 @@ int db_writeline(db_line* line,db_config* conf){
   if (line==NULL||conf==NULL) return RETOK;
   
   switch (conf->db_out_url->type) {
+#ifdef WITH_CURL
+  case url_http:
+  case url_https:
+  case url_ftp:
+#endif /* WITH CURL */
   case url_stdout:
   case url_stderr:
   case url_fd:
@@ -522,7 +729,7 @@ int db_writeline(db_line* line,db_config* conf){
        (conf->gzip_dbout && conf->db_gzout) ||
 #endif
        (conf->db_out!=NULL)) {
-      if (db_writeline_file(line,conf)==RETOK) {
+      if (db_writeline_file(line,conf,conf->db_out_url)==RETOK) {
 	return RETOK;
       }
     }
@@ -569,6 +776,15 @@ int db_close(db_config* conf)
     return RETFAIL;
     break;
   }
+#ifdef WITH_CURL
+  case url_http:
+  case url_https:
+  case url_ftp:
+    {
+      url_fclose(conf->db_out);
+      break;
+    }
+#endif /* WITH CURL */
 #ifdef WITH_PSQL
   case url_sql: {
     if (conf->db_out!=NULL) {
@@ -596,7 +812,7 @@ void free_db_line(db_line* dl)
     return;
   }
   
-#define checked_free(x) if(x!=NULL) { free(x); x=NULL; }
+#define checked_free(x) do { free(x); x=NULL; } while (0)
 
   checked_free(dl->md5);
   checked_free(dl->sha1);
@@ -605,10 +821,30 @@ void free_db_line(db_line* dl)
   checked_free(dl->filename);
   checked_free(dl->linkname);
   
+#ifdef WITH_MHASH
   checked_free(dl->crc32);
   checked_free(dl->crc32b);
   checked_free(dl->gost);
   checked_free(dl->haval);
+#endif
+  
+  checked_free(dl->sha256);
+  checked_free(dl->sha512);
+  checked_free(dl->whirlpool);
+
+  if (dl->acl)
+  {
+#ifdef WITH_ACL
+    free(dl->acl->acl_a);
+    free(dl->acl->acl_d);
+#endif
+  }
+  checked_free(dl->acl);
+  
+  if (dl->xattrs)
+    free(dl->xattrs->ents);
+  checked_free(dl->xattrs);
+  checked_free(dl->cntx);
 }
 const char* aide_key_5=CONFHMACKEY_05;
 const char* db_key_5=DBHMACKEY_05;
