@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/db_file.c,v 1.3 2003/08/18 13:06:30 rvdb Exp $
+ * $Header: /cvsroot/aide/aide/src/db_file.c,v 1.20 2006/11/24 14:07:32 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -32,7 +33,6 @@
 #include "db_file.h"
 #include "conf_yacc.h"
 #include "util.h"
-#include "aide.h"
 #include "db_sql.h" /* typedefs */
 #include "commandconf.h"
 /*for locale support*/
@@ -65,15 +65,23 @@ void handle_gzipped_input(int out,gzFile* gzp){
   error(200,"handle_gzipped_input(),%d\n",out);
   while(!gzeof(*gzp)){
     if((nread=gzread(*gzp,buf,ZBUFSIZE))<0){
-      error(0,_("gzread() failed:gzerr=%s!\n"),gzerror(*gzp,&err));
+      error(0,_("gzread() failed: gzerr=%s!\n"),gzerror(*gzp,&err));
       exit(1);
     } else {
-      /* gzread returns 0 even if uncompressed bytes were read*/
+      int tmp = 0;
+      
+      /* gzread returns 0 even if uncompressed bytes were read */
       if(nread==0){
-	write(out, buf,strlen((char*)buf));
+        tmp = strlen((char*)buf);
       } else {
-	write(out, buf,nread);
+        tmp = nread;
       }
+      if (write(out, buf,nread) != tmp)
+      {
+        error(0,_("write() failed: %m\n"));
+        exit(1);
+      }
+      
       error(240,"nread=%d,strlen(buf)=%d,errno=%s,gzerr=%s\n",
 	    nread,strlen((char*)buf),strerror(errno),
 	    gzerror(*gzp,&err));
@@ -137,9 +145,8 @@ int dofprintf( const char* s,...)
     retval=gzwrite(conf->db_gzout,temp,retval);
   }else{
 #endif
-    va_start(ap,s);
-    retval=vfprintf(conf->db_out,s,ap);
-    va_end(ap);
+    /* writing is ok with fwrite with curl.. */
+    retval=fwrite(temp,1,retval,conf->db_out);
 #ifdef WITH_ZLIB
   }
 #endif
@@ -245,13 +252,13 @@ int db_file_read_spec(int db){
     }
     
     case TDBSPEC : {
-      error(0,"Only one @@dbspec in inputdatabase.\n");
+      error(0,"Only one @@dbspec in input database.\n");
       return RETFAIL;
       break;
     }
     
     default : {
-      error(0,"Aide internal error while reading inputdatabase.\n");
+      error(0,"Aide internal error while reading input database.\n");
       return RETFAIL;
     }
     }
@@ -265,7 +272,7 @@ int db_file_read_spec(int db){
       conf->attr=1;
     }
   }
-  if (conf->attr==-1) {
+  if (conf->attr==DB_ATTR_UNDEF) {
     conf->attr=0;
     error(0,"Database does not have attr field.\nComparation may be incorrect\nGenerating attr-field from dbspec\nIt might be a good Idea to regenerate databases. Sorry.\n");
     for(i=0;i<conf->db_in_size;i++) {
@@ -286,7 +293,9 @@ char** db_readline_file(int db){
   int gotbegin_db=0;
   int gotend_db=0;
   int* domd=NULL;
+#ifdef WITH_MHASH
   MHASH* md=NULL;
+#endif
   char** oldmdstr=NULL;
   int* db_osize=0;
   DB_FIELD** db_order=NULL;
@@ -298,23 +307,30 @@ char** db_readline_file(int db){
 
   switch (db) {
   case DB_OLD: {
+#ifdef WITH_MHASH
     md=&(conf->dboldmd);
+#endif
     domd=&(conf->do_dboldmd);
     oldmdstr=&(conf->old_dboldmdstr);
+    
     db_osize=&(conf->db_in_size);
     db_order=&(conf->db_in_order);
     db_filep=&(conf->db_in);
     db_url=conf->db_in_url;
     db_lineno=&db_in_lineno;
+    
 #ifdef WITH_ZLIB
     db_gzp=&(conf->db_gzin);
 #endif
     break;
   }
   case DB_NEW: {
+#ifdef WITH_MHASH
     md=&(conf->dbnewmd);
+#endif
     domd=&(conf->do_dbnewmd);
     oldmdstr=&(conf->old_dbnewmdstr);
+    
     db_osize=&(conf->db_new_size);
     db_order=&(conf->db_new_order);
     db_filep=&(conf->db_new);
@@ -374,7 +390,7 @@ char** db_readline_file(int db){
     }
 
     if(FORCEDBMD&&!gotbegin_db){
-      error(0,"Database %i does not have checksum!\n");
+      error(0,"Database %i does not have checksum!\n",db);
       return NULL;
     }
 
@@ -385,20 +401,28 @@ char** db_readline_file(int db){
       
       switch (db_url->type) {
       case url_file : {
-	error(0,"Filedatabase must have one db_spec spesification\n");
+	error(0,"File database must have one db_spec specification\n");
 	break;
       }
 
       case url_stdin : {
-	error(0,"Pipedatabase must have one db_spec spesification\n");
+	error(0,"Pipe database must have one db_spec specification\n");
 	break;
       }
 
       case url_fd: {
-	error(0,"FD-database must have one db_spec spesification\n");
+	error(0,"FD database must have one db_spec specification\n");
 	break;
       }
-
+#ifdef WITH_CURL
+      case url_http:
+      case url_https:
+      case url_ftp: {
+	error(0,"CURL database must have one db_spec specification %i\n",token);
+	break;
+      }
+#endif
+	
       default : {
 	error(0,"db_readline_file():Unknown or unsupported db in type.\n");
 	
@@ -435,7 +459,7 @@ char** db_readline_file(int db){
       
     case TDBSPEC : {
       
-      error(0,"Databasefile can have only one db_spec.\nTrying to continue on line %i\n",*db_lineno);      
+      error(0,"Database file can have only one db_spec.\nTrying to continue on line %i\n",*db_lineno);      
       break;
     }
     case TNAME : {
@@ -486,7 +510,8 @@ char** db_readline_file(int db){
       if(token!=TSTRING){
 	error(0,_("Corrupt db. Checksum garbled\n"));
 	abort();
-      } else {
+      } else { /* FIXME: this probably isn't right */
+#ifdef WITH_MHASH
 	if(*md){
 	  byte* dig=NULL;
 	  char* digstr=NULL;
@@ -502,10 +527,13 @@ char** db_readline_file(int db){
 	    error(0,_("Db checksum mismatch for db:%i\n"),db);
 	    abort();
 	  }
-	}else {
+	}
+        else
+        {
 	  error(0,"@@end_db found without @@begin_db in db:%i\n",db);
 	  abort();
 	}
+#endif
       }
       token=db_scan();
       if(token!=TNEWLINE){
@@ -517,6 +545,10 @@ char** db_readline_file(int db){
 
     case TEND_DBNOMD : {
       gotend_db=1;
+      if(FORCEDBMD){
+        error(0,"Database %i does not have checksum!\n",db);
+	abort();
+      }
       break;
     }
 
@@ -555,12 +587,12 @@ char** db_readline_file(int db){
   
 
   /*
-   * If we don't get newline after reading all sells we print an error
+   * If we don't get newline after reading all cells we print an error
    */
   a=db_scan();
 
   if (a!=TNEWLINE&&a!=TEOF) {
-    error(0,"Newline expected in database. Readin until end of line\n");
+    error(0,"Newline expected in database. Reading until end of line\n");
     do {
       
       error(0,"Skipped value %s\n",dbtext);
@@ -582,6 +614,8 @@ int db_writechar(char* s,FILE* file,int i)
   char* r=NULL;
   int retval=0;
 
+  (void)file;
+  
   if(i) {
     dofprintf(" ");
   }
@@ -616,20 +650,38 @@ int db_writechar(char* s,FILE* file,int i)
 
 int db_writeint(long i,FILE* file,int a)
 {
+  (void)file;
+  
   if(a) {
     dofprintf(" ");
   }
   
   return dofprintf("%li",i);
   
+}
+int db_writelong(AIDE_OFF_TYPE i,FILE* file,int a)
+{
+  (void)file;
+  
+  if(a) {
+    dofprintf(" ");
+  }
+  
+#if AIDE_OFF_TYPE == off64_t
+  return dofprintf("%lli",i);
+#else
+  return dofprintf("%li",i);
+#endif
+  
 }
 
-int db_write_byte_base64(byte*data,size_t len,FILE* file,int i,int th,
-			 int attr )
+int db_write_byte_base64(byte*data,size_t len,FILE* file,int i,
+                         DB_ATTR_TYPE th, DB_ATTR_TYPE attr )
 {
   char* tmpstr=NULL;
   int retval=0;
   
+  (void)file;  
   
   if (data!=NULL&&th&attr) {
     tmpstr=encode_base64(data,len);
@@ -657,6 +709,8 @@ int db_write_time_base64(time_t i,FILE* file,int a)
   char* tmpstr=NULL;
   int retval=0;
 
+  (void)file;
+  
   if(a){
     dofprintf(" ");
   }
@@ -669,7 +723,7 @@ int db_write_time_base64(time_t i,FILE* file,int a)
 
   ptr=(char*)malloc(sizeof(char)*TIMEBUFSIZE);
   if (ptr==NULL) {
-    error(0,"\nCannot allocate memory..\n");
+    error(0,"\nCannot allocate memory.\n");
     abort();
   }
   memset((void*)ptr,0,sizeof(char)*TIMEBUFSIZE);
@@ -677,7 +731,7 @@ int db_write_time_base64(time_t i,FILE* file,int a)
   sprintf(ptr,"%li",i);
 
 
-  tmpstr=encode_base64(ptr,strlen(ptr));
+  tmpstr=encode_base64((byte *)ptr,strlen(ptr));
   retval=dofprintf(tmpstr);
   free(tmpstr);
   free(ptr);
@@ -688,6 +742,8 @@ int db_write_time_base64(time_t i,FILE* file,int a)
 
 int db_writeoct(long i, FILE* file,int a)
 {
+  (void)file;
+  
   if(a) {
     dofprintf(" ");
   }
@@ -755,7 +811,7 @@ int db_writespec_file(db_config* conf)
   }
   for(i=0;i<conf->db_out_size;i++){
     for(j=0;j<db_unknown;j++){
-      if(db_value[j]==conf->db_out_order[i]){
+      if((int)db_value[j]==(int)conf->db_out_order[i]){
 	retval=dofprintf("%s ",db_names[j]);
 	if(retval==0){
 	  return RETFAIL;
@@ -771,8 +827,9 @@ int db_writespec_file(db_config* conf)
   return RETOK;
 }
 
+int db_writeacl(acl_type* acl,FILE* file,int a)
+{
 #ifdef WITH_SUN_ACL
-int db_writeacl(acl_type* acl,FILE* file,int a){
   int i;
 
   if(a) {
@@ -790,13 +847,45 @@ int db_writeacl(acl_type* acl,FILE* file,int a){
 	      acl->acl[i].a_perm);
     }
   }
+#endif
+#ifdef WITH_POSIX_ACL
+  if(a) {
+    dofprintf(" ");
+  }
+  
+  if (acl==NULL) {
+    dofprintf("0");
+  } else {    
+    dofprintf("POSIX"); /* This is _very_ incompatible */
+
+    dofprintf(",");
+    if (acl->acl_a)
+      db_write_byte_base64((byte*)acl->acl_a, strlen(acl->acl_a), file,0,1,1);
+    else
+      dofprintf("0");
+    dofprintf(",");
+    if (acl->acl_d)
+      db_write_byte_base64((byte*)acl->acl_d, strlen(acl->acl_d), file,0,1,1);
+    else
+      dofprintf("0");
+  }
+#endif
+#ifndef WITH_ACL
+  if(a) { /* compat. */
+    dofprintf(" ");
+  }
+  
+  dofprintf("0");
+#endif
+  
   return RETOK;
 }
-#endif
 
-int db_writeline_file(db_line* line,db_config* conf){
+int db_writeline_file(db_line* line,db_config* conf, url_t* url){
   int i;
 
+  (void)url;
+  
   for(i=0;i<conf->db_out_size;i++){
     switch (conf->db_out_order[i]) {
     case db_filename : {
@@ -841,7 +930,7 @@ int db_writeline_file(db_line* line,db_config* conf){
       break;
     }
     case db_size : {
-      db_writeint(line->size,conf->db_out,i);
+      db_writelong(line->size,conf->db_out,i);
       break;
     }
     case db_md5 : {
@@ -906,9 +995,32 @@ int db_writeline_file(db_line* line,db_config* conf){
 			   DB_GOST,line->attr);
       break;
     }
+    case db_sha256 : {
+      db_write_byte_base64(line->sha256,
+			   HASH_SHA256_LEN,
+			   conf->db_out,i,
+			   DB_SHA256,line->attr);
+
+      break;
+    }
+    case db_sha512 : {
+      db_write_byte_base64(line->sha512,
+			   HASH_SHA512_LEN,
+			   conf->db_out,i,
+			   DB_SHA512,line->attr);
+
+      break;
+    }
+    case db_whirlpool : {
+      db_write_byte_base64(line->whirlpool,
+			   HASH_WHIRLPOOL_LEN,
+			   conf->db_out,i,
+			   DB_WHIRLPOOL,line->attr);
+
+      break;
+    }
     case db_attr : {
-      db_writeint(line->attr,
-		  conf->db_out,i);
+      db_writelong(line->attr, conf->db_out,i);
       break;
     }
 #ifdef WITH_ACL
@@ -917,6 +1029,36 @@ int db_writeline_file(db_line* line,db_config* conf){
       break;
     }
 #endif
+    case db_xattrs : {
+        xattr_node *xattr = NULL;
+        size_t num = 0;
+        
+        if (!line->xattrs)
+        {
+          db_writelong(0, conf->db_out, i);
+          break;
+        }
+        
+        db_writelong(line->xattrs->num, conf->db_out, i);
+        
+        xattr = line->xattrs->ents;
+        while (num < line->xattrs->num)
+        {
+          dofprintf(",");
+          db_writechar(xattr->key, conf->db_out, 0);
+          dofprintf(",");
+          db_write_byte_base64(xattr->val, xattr->vsz, conf->db_out, 0, 1, 1);
+          
+          ++xattr;
+          ++num;
+        }
+      break;
+    }
+    case db_selinux : {
+	db_write_byte_base64((byte*)line->cntx, line->cntx?strlen(line->cntx):0,
+                             conf->db_out, i, 1, 1);
+      break;
+    }
     case db_checkmask : {
       db_writeoct(line->attr,conf->db_out,i);
       break;
@@ -987,6 +1129,4 @@ int db_close_file(db_config* conf){
 
   return RETOK;
 }
-/*
-const char* aide_key_11=CONFHMACKEY_11;
-*/
+// vi: ts=8 sw=8
