@@ -1,7 +1,8 @@
 /* aide, Advanced Intrusion Detection Environment
+ * vi: ts=8 sw=8
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/do_md.c,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002,2005 Rami Lehti, Pablo Virolainen
+ * $Header: /cvsroot/aide/aide/src/do_md.c,v 1.21 2006/10/27 20:47:20 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +19,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
+
+#ifndef _POSIX_C_SOURCE
+#define _POSIX_C_SOURCE 200112L
+#endif
+
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,14 +42,12 @@
 #include "do_md.h"
 #include "report.h"
 #include "list.h"
-#include "aide.h"
 /*for locale support*/
 #include "locale-aide.h"
 /*for locale support*/
 
 
 /* This define should be somewhere else */
-#define READ_BLOCK_SIZE 16384
 #define READ_BLOCK_SIZE 16777216
 
 #ifdef WITH_MHASH
@@ -53,22 +58,21 @@
 #ifdef HAVE_MMAP
 #ifndef MAP_FAILED
 #define MAP_FAILED  (-1)
-#endif
+#endif /* MAP_FAILED */
 #define MMAP_BLOCK_SIZE 16777216
-#endif
+#endif /* HAVE_MMAP */
 
 /*
 #include <gcrypt.h>
 */
-void md_init_fail(const char* s,db_line* db,byte** hash,unsigned i) {
-  error(0,"Message diggest %s initialise failed\nDisabling %s for file %s\n",s,s,db->filename);
+void md_init_fail(const char* s,db_line* db,byte** hash,DB_ATTR_TYPE i) {
+  error(0,"Message digest %s initialise failed\nDisabling %s for file %s\n",s,s,db->filename);
   db->attr=db->attr&(~i);
   (*hash)=0;
 }
 
 void free_hashes(db_line* dl){
 
-  /* #define free_hash(a) if(dl->a!=NULL) {free(dl->a); dl->a=NULL;} */
 #define free_hash(a) dl->a=NULL
 
   free_hash(md5);
@@ -81,36 +85,28 @@ void free_hashes(db_line* dl){
   free_hash(gost);
   free_hash(crc32b);  
 #endif
-}
-
-list* do_md(list* file_lst,db_config* conf)
-{
-  abort();
-  return file_lst;
+  free_hash(sha256);
+  free_hash(sha512);
 }
 
 int stat_cmp(struct AIDE_STAT_TYPE* f1,struct AIDE_STAT_TYPE* f2) {
   if (f1==NULL || f2==NULL) {
     return RETFAIL;
   }
-#define stat_cmp_helper(n) (f1->n==f2->n)
+#define stat_cmp_helper(n,n2) ((f1->n!=f2->n)*n2)
 
-  if (stat_cmp_helper(st_ino)&&
-      stat_cmp_helper(st_mode)&&
-      stat_cmp_helper(st_nlink)&&
-      stat_cmp_helper(st_size)&&
-      stat_cmp_helper(st_mtime)&&
-      stat_cmp_helper(st_ctime)&&
-      stat_cmp_helper(st_blocks)&&
-      stat_cmp_helper(st_blksize)&&
-      stat_cmp_helper(st_rdev)&&
-      stat_cmp_helper(st_gid)&&
-      stat_cmp_helper(st_uid)&&
-      stat_cmp_helper(st_nlink)&&
-      stat_cmp_helper(st_dev)) {
-    return RETOK;
-  }
-  return RETFAIL;
+  return (stat_cmp_helper(st_ino,DB_INODE)|
+	  stat_cmp_helper(st_mode,DB_PERM)|
+	  stat_cmp_helper(st_nlink,DB_LNKCOUNT)|
+	  stat_cmp_helper(st_size,DB_SIZE)|
+	  stat_cmp_helper(st_mtime,DB_MTIME)|
+	  stat_cmp_helper(st_ctime,DB_CTIME)|
+	  stat_cmp_helper(st_blocks,DB_BCOUNT)|
+	  stat_cmp_helper(st_blksize,DB_BSIZE)|
+	  stat_cmp_helper(st_rdev,DB_RDEV)|
+	  stat_cmp_helper(st_gid,DB_GID)|
+	  stat_cmp_helper(st_uid,DB_UID)|
+	  stat_cmp_helper(st_dev,DB_DEV));
 }
 
 
@@ -124,22 +120,28 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
    */
   struct AIDE_STAT_TYPE fs;
   int sres=0;
+  int stat_diff,filedes;
+
+  error(255,"calc_md called\n");
 #ifdef _PARAMETER_CHECK_
   if (line==NULL) {
     abort();
   }
 #endif  
 
-  int filedes=open(line->filename,O_RDONLY);
+#ifdef HAVE_O_NOATIME
+  filedes=open(line->filename,O_RDONLY|O_NOATIME);
+  if(filedes==-1)
+#endif
+    filedes=open(line->filename,O_RDONLY);
 
-  error(255,"calc_md called\n");
   if (filedes==-1) {
     char* er=strerror(errno);
     if (er!=NULL) {
-      error(5,"do_md():open() for %s failed:%s\n",
+      error(3,"do_md(): open() for %s failed: %s\n",
 	    line->filename,er);
     } else {
-      error(5,"do_md():open() for %s failed:%i\n",
+      error(3,"do_md(): open() for %s failed: %i\n",
 	    line->filename,errno);
     }
     /*
@@ -150,8 +152,17 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
   }
   
   sres=AIDE_FSTAT_FUNC(filedes,&fs);
+  if(!(line->attr&DB_RDEV))
+	  fs.st_rdev=0;
   
-  if (stat_cmp(&fs,old_fs)==RETOK) {
+#ifdef HAVE_POSIX_FADVICE
+  if (posix_fadvise(filedes,0,fs.st_size,POSIX_FADV_NOREUSE)!=0) {
+	error(255,"posix_fadvise error %s\n",strerror(errno));
+  } else {
+	error(255,"posix_fadvise(%i,0,%i,POSIX_FADV_NOREUSE) ok\n",filedes,fs.st_size);
+  }
+#endif
+  if ((stat_diff=stat_cmp(&fs,old_fs))==RETOK) {
     /*
       Now we have a 'valid' filehandle to read from a file.
      */
@@ -171,18 +182,26 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
       /* in mmap branch r_size is used as size remaining */
       while(r_size>0){
 	if(r_size<MMAP_BLOCK_SIZE){
+#ifdef __hpux
+	  buf = mmap(0,r_size,PROT_READ,MAP_PRIVATE,filedes,curpos);
+#else
 	  buf = mmap(0,r_size,PROT_READ,MAP_SHARED,filedes,curpos);
+#endif
 	  curpos+=r_size;
 	  size=r_size;
 	  r_size=0;
 	}else {
+#ifdef __hpux
+	  buf = mmap(0,MMAP_BLOCK_SIZE,PROT_READ,MAP_PRIVATE,filedes,curpos);
+#else
 	  buf = mmap(0,MMAP_BLOCK_SIZE,PROT_READ,MAP_SHARED,filedes,curpos);
+#endif
 	  curpos+=MMAP_BLOCK_SIZE;
 	  size=MMAP_BLOCK_SIZE;
 	  r_size-=MMAP_BLOCK_SIZE;
 	}
 	if ( buf == MAP_FAILED ) {
-	  error(0,"error mmap'ing %s\n", line->filename);
+	  error(0,"error mmap'ing %s: %s\n", line->filename,strerror(errno));
 	  close(filedes);
 	  close_md(&mdc);
 	  return;
@@ -202,7 +221,7 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
 #if READ_BLOCK_SIZE>SSIZE_MAX
 #error "READ_BLOCK_SIZE" is too large. Max value is SSIZE_MAX, and current is READ_BLOCK_SIZE
 #endif
-      while ((size=read(filedes,buf,READ_BLOCK_SIZE))>0) {
+      while ((size=TEMP_FAILURE_RETRY(read(filedes,buf,READ_BLOCK_SIZE)))>0) {
 	if (update_md(&mdc,buf,size)!=RETOK) {
 	  error(0,"Message digest failed during update\n");
 	  close_md(&mdc);
@@ -216,18 +235,25 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
       md2line(&mdc,line);
       
     } else {
-      error(5,"Message diggest initialization failed.\n");
+      error(3,"Message digest initialization failed.\n");
       no_hash(line);
       close(filedes);
       return;
     }
   } else {
+    unsigned i;
     /*
       Something just wasn't correct, so no hash calculated.
     */
     
     error(5,"File %s was changed so that hash cannot be calculated for it\n"
 	  ,line->filename);
+
+    for(i=0;i<db_unknown;i++) {
+      if (((1<<i)&stat_diff)!=0) {
+	error(5,"Attribute %s has been changed\n",db_names[i]);
+      }
+    }
     
     no_hash(line);
     close(filedes);
@@ -239,7 +265,7 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
 
 void fs2db_line(struct AIDE_STAT_TYPE* fs,db_line* line) {
   
-  if(DB_INODE&line->attr){
+  if(DB_INODE&line->attr||DB_CHECKINODE&line->attr){
     line->inode=fs->st_ino;
   } else {
     line->inode=0;
@@ -301,8 +327,58 @@ void fs2db_line(struct AIDE_STAT_TYPE* fs,db_line* line) {
   
 }
 
-#ifdef WITH_ACL
 void acl2line(db_line* line) {
+  acl_type *ret = NULL;
+  
+#ifdef WITH_POSIX_ACL
+  if(DB_ACL&line->attr) {
+    acl_t acl_a;
+    acl_t acl_d;
+    char *tmp = NULL;
+
+    acl_a = acl_get_file(line->filename, ACL_TYPE_ACCESS);
+    acl_d = acl_get_file(line->filename, ACL_TYPE_DEFAULT);
+    if ((acl_a == NULL) && (errno == ENOTSUP))
+      return;
+    if (acl_a == NULL)
+      error(0, "Tried to read access ACL on %s but failed with: %m\n",
+            line->filename);
+    if ((acl_d == NULL) && (errno != EACCES)) /* ignore DEFAULT on files */
+    {
+      acl_free(acl_a);
+      error(0, "Tried to read default ACL on %s but failed with: %m\n",
+            line->filename);
+    }
+
+    /* assume memory allocs work, like rest of AIDE code... */
+    ret = malloc(sizeof(acl_type));
+
+    /* use tmp, so free() can be called instead of acl_free() */
+    tmp = acl_to_text(acl_a, NULL);
+    if (!tmp || !*tmp)
+      ret->acl_a = NULL;
+    else
+      ret->acl_a = strdup(tmp);
+    acl_free(tmp);
+
+    if (!acl_d)
+      ret->acl_d = NULL;
+    else
+    {
+      tmp = acl_to_text(acl_d, NULL);
+      if (!tmp || !*tmp)
+        ret->acl_d = NULL;
+      else
+        ret->acl_d = strdup(tmp);
+      acl_free(tmp);
+    }
+
+    acl_free(acl_a);
+    acl_free(acl_d);
+  }
+  line->acl = ret;
+#endif  
+#ifdef WITH_SUN_ACL
   if(DB_ACL&line->attr) { /* There might be a bug here. */
     int res;
     line->acl=malloc(sizeof(acl_type));
@@ -329,8 +405,8 @@ void acl2line(db_line* line) {
   }else{
     line->acl=NULL;
   }
-}
 #endif
+}
 
 void no_hash(db_line* line) {
   line->attr&=~DB_HASHES;
