@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti and Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/error.c,v 1.2 2003/08/01 19:28:55 madhack Exp $
+ * $Header: /cvsroot/aide/aide/src/error.c,v 1.8 2006/10/27 18:54:29 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "../config.h"
+#include "aide.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -31,11 +31,11 @@
 
 #include "report.h"
 #include "list.h"
-#include "aide.h"
 #include "be.h"
 /*for locale support*/
 #include "locale-aide.h"
 /*for locale support*/
+#include "util.h"
 
 int cmp_url(url_t* url1,url_t* url2){
   
@@ -47,6 +47,7 @@ int error_init(url_t* url,int initial)
 {
   list* r=NULL;
   FILE* fh=NULL;
+	int   sfac;
   
   if (url->type==url_database) {
     conf->report_db++;
@@ -59,7 +60,8 @@ int error_init(url_t* url,int initial)
 #ifdef HAVE_SYSLOG
       conf->initial_report_url=url;
       conf->initial_report_fd=NULL;
-      openlog(AIDE_IDENT,AIDE_LOGOPT, AIDE_SYSLOG_FACILITY);
+      sfac=syslog_facility_lookup(url->value);
+      openlog(AIDE_IDENT,AIDE_LOGOPT, sfac);
       
       return RETOK;
 #endif
@@ -99,9 +101,12 @@ int error_init(url_t* url,int initial)
     /* If you add support for facility changing in config 
        consider multiple calls of openlog.
        This openlog MUST NOT mess up initial errorsto openlog.
+       RvdB 22/1/2006: the 2 openlog calls where the same before my
+       change, and they are still the same, I assume I did not brake anything
     */
+    sfac=syslog_facility_lookup(url->value);
     if(conf->report_syslog<2)
-      openlog(AIDE_IDENT,AIDE_LOGOPT, AIDE_SYSLOG_FACILITY);
+      openlog(AIDE_IDENT,AIDE_LOGOPT, sfac);
 
     return RETOK;
 #endif
@@ -157,7 +162,13 @@ void error(int errorlevel,char* error_msg,...)
     }
 #ifdef HAVE_SYSLOG
     if(conf->initial_report_url->type==url_syslog){
+#ifdef HAVE_VSYSLOG
       vsyslog(SYSLOG_PRIORITY,error_msg,ap);
+#else
+			char buf[1024];
+			vsnprintf(buf,1024,error_msg,ap);
+			syslog(SYSLOG_PRIORITY,"%s",buf);
+#endif
       va_end(ap);
       return;
     }
@@ -169,9 +180,17 @@ void error(int errorlevel,char* error_msg,...)
 
 #ifdef HAVE_SYSLOG
   if (conf->report_syslog!=0) {
+#ifdef HAVE_VSYSLOG
     va_start(ap,error_msg);
-    syslog(SYSLOG_PRIORITY,error_msg,ap);
+    vsyslog(SYSLOG_PRIORITY,error_msg,ap);
     va_end(ap);
+#else
+		char buf[1024];
+    va_start(ap,error_msg);
+		vsnprintf(buf,1024,error_msg,ap);
+    va_end(ap);
+		syslog(SYSLOG_PRIORITY,"%s",buf);
+#endif
   }
 #endif
 
