@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/gen_list.c,v 1.4 2003/11/04 15:39:48 rvdb Exp $
+ * $Header: /cvsroot/aide/aide/src/gen_list.c,v 1.30 2006/12/08 22:00:37 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,8 +18,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <string.h>
 #include "aide.h"
+	       
+#include <string.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <dirent.h>
@@ -27,13 +28,10 @@
 #include <limits.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#ifdef HAVE_USTAT
-#include <ustat.h>
-#endif
 #include <errno.h>
 #include <time.h>
+
 #include "report.h"
-#include "config.h"
 #include "gnu_regex.h"
 #include "list.h"
 #include "gen_list.h"
@@ -52,10 +50,8 @@
 #ifdef WITH_MHASH
 #include <mhash.h>
 #endif
-/*
-#include <gcrypt.h>
-*/
 #include "md.h"
+#include "do_md.h"
 
 void hsymlnk(db_line* line);
 void fs2db_line(struct AIDE_STAT_TYPE* fs,db_line* line);
@@ -65,57 +61,37 @@ void no_hash(db_line* line);
 char* strrxtok(char* rx)
 {
   char*p=NULL;
-  int i=0;
-  int j=1;
-  int last_backslash=0;
-  int lastslash_in_p=1;
+  size_t i=0;
 
   /* The following code assumes that the first character is a slash */
-  int lastslash=0;
+  size_t lastslash=1;
 
-  p=(char*)malloc(sizeof(char)*strlen(rx)+1);
+  p=strdup(rx);
   p[0]='/';
 
-  for(i=1;i<strlen(rx);i++){
-    switch(rx[i])
+  for(i=1;i<strlen(p);i++){
+    switch(p[i])
       {
       case '/':
-	if(last_backslash){
-  		strncpy(p+j,rx+last_backslash+1,i-last_backslash);
-		j+=i-last_backslash;
-	}else{
-		strncpy(p+j,rx+lastslash+1,i-lastslash);
-		j+=i-lastslash;
-	}
-		
 	lastslash=i;
-	lastslash_in_p=j-1;
-
 	break;
       case '(':
       case '^':
       case '$':
       case '*':
-      case '.':
       case '[':
-	i=strlen(rx);
+	i=strlen(p);
 	break;
       case '\\':
-	last_backslash=i;
-	strncpy(p+j,rx+lastslash+1,i-lastslash);
-	j+=i-lastslash;
-	/* Skip the next character */
+	strcpy(p+i,p+i+1);
 	i++;
-	/* Don't copy the backslash */
-	j--;
 	break;
       default:
 	break;
       }
   }
 
-  p[lastslash_in_p]='\0';
-
+  p[lastslash]='\0';
 
   return p;
 }
@@ -123,8 +99,8 @@ char* strrxtok(char* rx)
 char* strlastslash(char*str)
 {
   char* p=NULL;
-  int lastslash=1;
-  int i=0;
+  size_t lastslash=1;
+  size_t i=0;
 
   for(i=1;i<strlen(str);i++){
     if(str[i]=='/'){
@@ -175,6 +151,44 @@ int treedepth(seltree* node)
   return depth;
 }
 
+/* This function returns a node with the same inode value as the 'file' */
+/* The only place it is used is in add_file_to_tree() function */
+static seltree* get_seltree_inode(seltree* tree, db_line* file, int db)
+{
+  seltree* node=NULL;
+  list* r=NULL;
+  char* tmp=NULL;
+
+  if(tree==NULL){
+    return NULL;
+  }
+
+  /* found the match */
+  if((db == DB_NEW &&
+      tree->new_data != NULL &&
+      file->inode == tree->new_data->inode) ||
+     (db == DB_OLD &&
+      tree->old_data != NULL &&
+      file->inode == tree->old_data->inode)) {
+    return tree;
+  }
+
+  /* tmp is the directory of the file->filename */
+  tmp=strgetndirname(file->filename,treedepth(tree)+1);
+  for(r=tree->childs;r;r=r->next){
+    /* We are interested only in files with the same regexp specification */
+    if(strlen(tmp) == strlen(file->filename) ||
+       strncmp(((seltree*)r->data)->path,tmp,strlen(tmp)+1)==0){
+      node=get_seltree_inode((seltree*)r->data,file,db);
+      if(node!=NULL){
+	break;
+      }
+    }
+  }
+  free(tmp);
+  return node;
+}
+
 seltree* get_seltree_node(seltree* tree,char* path)
 {
   seltree* node=NULL;
@@ -209,8 +223,7 @@ void copy_rule_ref(seltree* node, rx_rule* r)
 {
     if( r!=NULL ){
         node->conf_lineno = r->conf_lineno;  
-        node->rx=(char*)malloc(strlen(r->rx)+1);
-        strcpy(node->rx,r->rx);
+        node->rx=strdup(r->rx);
     } else {
         node->conf_lineno = -1;
         node->rx=NULL;
@@ -245,13 +258,17 @@ seltree* new_seltree_node(
     if(isrx){
       parent=get_seltree_node(tree,tmprxtok);
     }else {
-      parent=get_seltree_node(tree,strlastslash(path));
+      char* dirn=strlastslash(path);
+      parent=get_seltree_node(tree,dirn);
+      free(dirn);
     }      
     if(parent==NULL){
       if(isrx){
 	parent=new_seltree_node(tree,tmprxtok,isrx,r);
       }else {
-	parent=new_seltree_node(tree,strlastslash(path),isrx,r);
+        char* dirn=strlastslash(path);
+        parent=new_seltree_node(tree,dirn,isrx,r);
+        free(dirn);
       }
     }
     free(tmprxtok);
@@ -278,12 +295,15 @@ void gen_seltree(list* rxlist,seltree* tree,char type)
     
     rxtok=strrxtok(curr_rule->rx);
     curnode=get_seltree_node(tree,rxtok);
+
     if(curnode==NULL){
       curnode=new_seltree_node(tree,rxtok,1,curr_rule);
     }
+
+    error(240,"Handling %s with %c \"%s\" with node \"%s\"\n",rxtok,type,curr_rule->rx,curnode->path);
+	
     
-    /* We have to add '^' to the first charaster of string... 
-     *
+    /* We have to add '^' to the first character of string... 
      */
 
     data=(char*)malloc(strlen(curr_rule->rx)+1+1);
@@ -332,11 +352,136 @@ void gen_seltree(list* rxlist,seltree* tree,char type)
      */
       free(rxtok);
   }
-
-
 }
 
-list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
+static xattrs_type *xattr_new(void)
+{
+  xattrs_type *ret = NULL;
+
+  ret = malloc(sizeof(xattrs_type));
+  ret->num = 0;
+  ret->sz  = 2;
+  ret->ents = malloc(sizeof(xattr_node) * ret->sz);
+
+  return (ret);
+}
+
+static void *xzmemdup(const void *ptr, size_t len)
+{ /* always keeps a 0 at the end... */
+  void *ret = NULL;
+
+  ret = malloc(len+1);
+  memcpy(ret, ptr, len);
+  ((char*)ret)[len] = 0;
+  
+  return (ret);
+}
+
+static void xattr_add(xattrs_type *xattrs,
+                      const char *key, const char *val, size_t vsz)
+{
+  if (xattrs->num >= xattrs->sz)
+  {
+    xattrs->sz <<= 1;
+    xattrs->ents = realloc(xattrs->ents, sizeof(xattr_node) * xattrs->sz);
+  }
+
+  xattrs->ents[xattrs->num].key = strdup(key);
+  xattrs->ents[xattrs->num].val = xzmemdup(val, vsz);
+  xattrs->ents[xattrs->num].vsz = vsz;
+
+  xattrs->num += 1;
+}
+
+/* should be in do_md ? */
+static void xattrs2line(db_line *line)
+{ /* get all generic user xattrs. */
+#ifdef WITH_XATTR
+  xattrs_type *xattrs = NULL;
+  static ssize_t xsz = 1024;
+  static char *xatrs = NULL;
+  ssize_t xret = -1;
+
+  if (!(DB_XATTRS&line->attr))
+    return;
+  
+  /* assume memory allocs work, like rest of AIDE code... */
+  if (!xatrs) xatrs = malloc(xsz);
+  
+  while (((xret = llistxattr(line->filename, xatrs, xsz)) == -1) &&
+         (errno == ERANGE))
+  {
+    xsz <<= 1;
+    xatrs = realloc(xatrs, xsz);
+  }
+
+  if ((xret == -1) && (errno == ENOSYS))
+  { /* do nothing */ }
+  else if (xret == -1)
+    error(0, "listxattrs failed for %s:%m\n", line->filename);
+  else if (xret)
+  {
+    const char *attr = xatrs;
+    static ssize_t asz = 1024;
+    static char *val = NULL;
+
+    if (!val) val = malloc(asz);
+    
+    xattrs = xattr_new();
+  
+    while (xret > 0)
+    {
+      size_t len = strlen(attr);
+      ssize_t aret = 0;
+      
+      if (strncmp(attr, "user.", strlen("user.")) &&
+          strncmp(attr, "root.", strlen("root.")))
+        goto next_attr; /* only store normal xattrs, and SELinux */
+      
+      while (((aret = getxattr(line->filename, attr, val, asz)) == -1) &&
+             (errno == ERANGE))
+      {
+        asz <<= 1;
+        val = realloc (val, asz);
+      }
+      
+      if (aret != -1)
+        xattr_add(xattrs, attr, val, aret);
+      else if (errno != ENOATTR)
+        error(0, "getxattr failed for %s:%m\n", line->filename);
+      
+     next_attr:
+      attr += len + 1;
+      xret -= len + 1;
+    }
+  }
+
+  line->xattrs = xattrs;
+#endif
+}
+
+/* should be in do_md ? */
+static void selinux2line(db_line *line)
+{
+#ifdef WITH_SELINUX
+  char *cntx = NULL;
+
+  if (!(DB_SELINUX&line->attr))
+    return;
+
+  if (lgetfilecon_raw(line->filename, &cntx) == -1)
+  {
+    error(0, "lgetfilecon_raw failed for %s:%m\n", line->filename);
+    return;
+  }
+
+  line->cntx = strdup(cntx);
+  
+  freecon(cntx);
+#endif
+}
+
+list* add_file_to_list(list* listp,char*filename,DB_ATTR_TYPE attr,int* addok)
 {
   db_line* fil=NULL;
   time_t cur_time;
@@ -365,6 +510,8 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     *addok=RETFAIL;
     return listp;
   } 
+  if(!(fil->attr&DB_RDEV))
+    fs.st_rdev=0;
   
   cur_time=time(NULL);
 
@@ -388,27 +535,6 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     }
   }
   
-#ifdef HAVE_USTAT
-  /* 
-   * This should not use ustat and should be implemented otherwise
-   * since ustat is not universally known function
-   * check find(1) for clues
-   */
-  /*
-    Here we should check if we need to add it..
-   */  
-  
-  {
-    struct ustat buf;
-    if (ustat(fs.st_dev, &buf) != 0 || buf.f_fname[0]==0) {
-      
-    } else {
-      
-    }
-
-  }
-#endif
-  
   fil->perm_o=fs.st_mode;
   fil->size_o=fs.st_size;
 
@@ -421,15 +547,17 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     if(conf->no_acl_on_symlinks!=1) {
       fil->attr&=(~DB_ACL);
     }
-#endif   
-
+#endif
+    
     if(conf->warn_dead_symlinks==1) {
       struct AIDE_STAT_TYPE fs;
       int sres;
       sres=AIDE_STAT_FUNC(fil->filename,&fs);
       if (sres!=0) {
-	error(5,"Dead symlink detected at %s\n",fil->filename);
+	error(4,"Dead symlink detected at %s\n",fil->filename);
       }
+      if(!(fil->attr&DB_RDEV))
+	fs.st_rdev=0;
     }
     
     if(conf->symlinks_found==0){
@@ -476,12 +604,12 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     fil->attr|=DB_LINKNAME;
   }else {
     fil->linkname=NULL;
-    /* Just remove linkname avaivilibity bit from this entry */
+    /* Just remove linkname availability bit from this entry */
     fil->attr&=(~DB_LINKNAME); 
   }
 
   
-  if(DB_INODE&fil->attr){
+  if(DB_INODE&fil->attr||DB_CHECKINODE&fil->attr){
     fil->inode=fs.st_ino;
   } else {
     fil->inode=0;
@@ -541,54 +669,28 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     fil->bcount=0;
   }
 
-#ifdef WITH_ACL
-  if(DB_ACL&fil->attr) { /* There might be a bug here. */
-    int res;
-    fil->acl=malloc(sizeof(acl_type));
-    fil->acl->entries=acl(fil->filename,GETACLCNT,0,NULL);
-    if (fil->acl->entries==-1) {
-      char* er=strerror(errno);
-      fil->acl->entries=0;
-      if (er==NULL) {
-	error(0,"ACL query failed for %s. strerror failed for %i\n",fil->filename,errno);
-      } else {
-	error(0,"ACL query failed for %s:%s\n",fil->filename,er);
-      }
-    } else {
-      fil->acl->acl=malloc(sizeof(aclent_t)*fil->acl->entries);
-      res=acl(fil->filename,GETACL,fil->acl->entries,fil->acl->acl);
-      if (res==-1) {
-	error(0,"ACL error %s\n",strerror(errno));
-      } else {
-	if (res!=fil->acl->entries) {
-	  error(0,"Tried to read %i acl but got %i\n",fil->acl->entries,res);
-	}
-      }
-    }
-  }else{
-    fil->acl=NULL;
-  }
-
-#endif
   
   if(S_ISDIR(fs.st_mode)||S_ISCHR(fs.st_mode)
      ||S_ISBLK(fs.st_mode)||S_ISFIFO(fs.st_mode)
      ||S_ISLNK(fs.st_mode)||S_ISSOCK(fs.st_mode)){
     
-    fil->attr&=(~DB_MD5)&(~DB_SHA1)&(~DB_RMD160)&(~DB_TIGER);
+    fil->attr&=(~DB_MD5)&(~DB_SHA1)&(~DB_SHA256)&(~DB_SHA512)&(~DB_RMD160)&(~DB_TIGER)&(~DB_CRC32)&(~DB_HAVAL);
     
     fil->md5=NULL;
     fil->sha1=NULL;
+    fil->sha256=NULL;
+    fil->sha512=NULL;
     fil->tiger=NULL;
     fil->rmd160=NULL;
+    fil->haval=NULL;
+    fil->crc32=NULL;
 #ifdef WITH_MHASH
 
-    fil->attr&=(~DB_CRC32)&(~DB_HAVAL)&(~DB_CRC32B);
+    fil->attr&=(~DB_CRC32B)&(~DB_GOST)&(~DB_WHIRLPOOL);
 
-    fil->crc32=NULL;
     fil->crc32b=NULL;
-    fil->haval=NULL;
     fil->gost=NULL;
+    fil->whirlpool=NULL;
 #endif
   }else {
     /* 1 if needs to be set
@@ -597,28 +699,61 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     fil->sha1=DB_SHA1&fil->attr?(byte*)"":NULL;
     fil->tiger=DB_TIGER&fil->attr?(byte*)"":NULL;
     fil->rmd160=DB_RMD160&fil->attr?(byte*)"":NULL;
-#ifdef WITH_MHASH
     fil->crc32=DB_CRC32&fil->attr?(byte*)"":NULL;
+    fil->haval=DB_HAVAL&fil->attr?(byte*)"":NULL;
+#ifdef WITH_MHASH
     fil->crc32b=DB_CRC32B&fil->attr?(byte*)"":NULL;
     fil->gost=DB_GOST&fil->attr?(byte*)"":NULL;
-    fil->haval=DB_HAVAL&fil->attr?(byte*)"":NULL;
+    fil->whirlpool=DB_WHIRLPOOL&fil->attr?(byte*)"":NULL;
 #endif
   }
+
+  xattrs2line(fil); /* NOTE ... this is a lie, code never gets here */
+  
   listp=list_append(listp,(void*)fil);
 
   *addok=RETOK;
   return listp;
 }
 
-int check_list_for_match(list* rxrlist,char* text,int* attr)
+int check_list_for_match(list* rxrlist,char* text,DB_ATTR_TYPE* attr)
 {
   list* r=NULL;
   int retval=1;
   for(r=rxrlist;r;r=r->next){
     if((retval=regexec((regex_t*)((rx_rule*)r->data)->crx,text,0,0,0))==0){
       *attr=((rx_rule*)r->data)->attr;
-        error(231,"Matches rule from line #%ld: %s\n",((rx_rule*)r->data)->conf_lineno,((rx_rule*)r->data)->rx);
+        error(231,"\"%s\" matches rule from line #%ld: %s\n",text,((rx_rule*)r->data)->conf_lineno,((rx_rule*)r->data)->rx);
       break;
+    } else {
+	error(232,"\"%s\" doesn't match rule from line #%ld: %s\n",text,((rx_rule*)r->data)->conf_lineno,((rx_rule*)r->data)->rx);
+    }
+  }
+  return retval;
+}
+
+//this is used to check if $text if equal to a node in $rxrlist
+//should be used to check equ_rx_lst only
+int check_list_for_equal(list* rxrlist,char* text,DB_ATTR_TYPE* attr)
+{
+  list* r=NULL;
+  int retval=1;
+  char *temp;
+
+  for(r=rxrlist;r;r=r->next){
+    temp=((rx_rule*)r->data)->rx;
+    
+    //FIXME, if rx not begin with ^, may need to do something else
+    if(temp[0]=='^') //^ is for reg exp, we can ignore this character
+      temp++;
+
+    //we don't need to worry about buff-overflow, so strcmp is safe
+    if((retval=strcmp(temp, text))==0){
+      *attr=((rx_rule*)r->data)->attr;
+      error(231,"\"%s\" matches string from line #%ld: %s\n",text,((rx_rule*)r->data)->conf_lineno,((rx_rule*)r->data)->rx);
+      break;
+    } else {
+      error(231,"\"%s\" doesn't match string from line #%ld: %s\n",text,((rx_rule*)r->data)->conf_lineno,((rx_rule*)r->data)->rx);
     }
   }
   return retval;
@@ -634,10 +769,13 @@ int check_list_for_match(list* rxrlist,char* text,int* attr)
  * 2,  if a equals rule was matched
  * retval if no rule was matched.
  * retval&3 if no rule was matched and first in the recursion
- *
+ * to keep state revat is orred with:
+ * 4,  matched deeper on equ rule
+ * 8,  matched deeper on sel rule
+ *16,  this is a recursed call
  */    
 
-int check_node_for_match(seltree*node,char*text,int retval,int* attr)
+int check_node_for_match(seltree*node,char*text,int retval,DB_ATTR_TYPE* attr)
 {
   int top=0;
   
@@ -650,15 +788,22 @@ int check_node_for_match(seltree*node,char*text,int retval,int* attr)
   if(!((retval&16)==16)){
     retval|=16;
     top=1;
-  } else{
+  } else {
     top=0;
   }
-    
+  
   /* if no deeper match found */
   if(!((retval&8)==8)&&!((retval&4)==4)){
-  	if(!check_list_for_match(node->equ_rx_lst,text,attr)){
-    		retval|=(2|4);
-  	};
+    if(!check_list_for_match(node->equ_rx_lst,text,attr)){
+      /*
+	Zhi Wen Wong added this line to fix bug that equ not work for 
+	compare
+	if we do "=/bin", we should only check /bin
+	so, /bin/bash or /bin/something should return 0 as neg
+      */
+      if(!check_list_for_equal(node->equ_rx_lst,text,attr))
+	retval|=(2|4);
+    };
   };
   /* We'll use retval to pass information on whether to recurse 
    * the dir or not */
@@ -688,7 +833,7 @@ int check_node_for_match(seltree*node,char*text,int retval,int* attr)
   return retval;
 }
 
-list* traverse_tree(seltree* tree,list* file_lst,int attr)
+list* traverse_tree(seltree* tree,list* file_lst,DB_ATTR_TYPE attr)
 {
   list* r=NULL;
   seltree* a=NULL;
@@ -699,7 +844,7 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
   int addfile=0;
   char* fullname=NULL;
   int e=0;
-  int matchattr=attr;
+  DB_ATTR_TYPE matchattr=attr;
 #  ifndef HAVE_READDIR_R
   long td=-1;
 #  endif
@@ -715,7 +860,7 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
   }
 
   if(!(dirh=opendir(tree->path))){
-    error(5,"traverse_tree():%s: %s\n", strerror(errno),tree->path);
+    error(3,"traverse_tree():%s: %s\n", strerror(errno),tree->path);
     return file_lst;
   }
   
@@ -834,6 +979,31 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
   return file_lst;
 }
 
+void print_tree(seltree* tree) {
+  
+  list* r;
+  rx_rule* rxc;
+  error(245,"tree: \"%s\"\n",tree->path);
+
+  for(r=tree->sel_rx_lst;r!=NULL;r=r->next) {
+	rxc=r->data;
+	error(246,"%i\t%s\n",rxc->conf_lineno,rxc->rx);
+  }
+  for(r=tree->equ_rx_lst;r!=NULL;r=r->next) {
+        rxc=r->data;
+        error(246,"%i=\t%s\n",rxc->conf_lineno,rxc->rx);
+  }
+  
+  for(r=tree->neg_rx_lst;r!=NULL;r=r->next) {
+	  rxc=r->data;
+	  error(246,"%i!\t%s\n",rxc->conf_lineno,rxc->rx);
+  }
+  
+  for(r=tree->childs;r!=NULL;r=r->next) {
+	print_tree(r->data);
+  }
+}
+
 seltree* gen_tree(list* prxlist,list* nrxlist,list* erxlist)
 {
   seltree* tree=NULL;
@@ -844,6 +1014,8 @@ seltree* gen_tree(list* prxlist,list* nrxlist,list* erxlist)
   gen_seltree(nrxlist,tree,'n');
   gen_seltree(erxlist,tree,'e');
 
+  print_tree(tree);
+
   return tree;
 }
 
@@ -868,14 +1040,13 @@ list* gen_list(list* prxlist,list* nrxlist,list* erxlist)
  * strip_dbline()
  * strips given dbline
  */
-void strip_dbline(db_line* line,int attr)
+void strip_dbline(db_line* line,DB_ATTR_TYPE attr)
 {
-#define checked_free(x) if(x!=NULL) free(x)
+#define checked_free(x) do { free(x); x=NULL; } while (0)
 
   /* filename is always needed, hence it is never stripped */
   if(!(attr&DB_LINKNAME)){
     checked_free(line->linkname);
-    line->linkname=NULL;
   }
   if(!(attr&DB_PERM)){
     line->perm=0;
@@ -901,7 +1072,7 @@ void strip_dbline(db_line* line,int attr)
   if(!(attr&DB_LNKCOUNT)){
     line->nlink=0;
   }
-  if(!(attr&DB_SIZE)){
+  if(!(attr&DB_SIZE)&&!(attr&DB_SIZEG)){
     line->size=0;
   }
   if(!(attr&DB_BCOUNT)){
@@ -910,36 +1081,59 @@ void strip_dbline(db_line* line,int attr)
 
   if(!(attr&DB_MD5)){
     checked_free(line->md5);
-    line->md5=NULL;
   }
   if(!(attr&DB_SHA1)){
     checked_free(line->sha1);
-    line->sha1=NULL;
   }
   if(!(attr&DB_RMD160)){
     checked_free(line->rmd160);
-    line->rmd160=NULL;
   }
   if(!(attr&DB_TIGER)){
     checked_free(line->tiger);
-    line->tiger=NULL;
-  }
-#ifdef WITH_MHASH
-  if(!(attr&DB_CRC32)){
-    checked_free(line->crc32);
-    line->crc32=NULL;
-  }
-  if(!(attr&DB_CRC32B)){
-    checked_free(line->crc32b);
-    line->crc32b=NULL;
-  }
-  if(!(attr&DB_GOST)){
-    checked_free(line->gost);
-    line->gost=NULL;
   }
   if(!(attr&DB_HAVAL)){
     checked_free(line->haval);
-    line->haval=NULL;
+  }
+  if(!(attr&DB_CRC32)){
+    checked_free(line->crc32);
+  }
+#ifdef WITH_MHASH
+  if(!(attr&DB_CRC32B)){
+    checked_free(line->crc32b);
+  }
+  if(!(attr&DB_GOST)){
+    checked_free(line->gost);
+  }
+  if(!(attr&DB_WHIRLPOOL)){
+    checked_free(line->whirlpool);
+  }
+#endif
+  if(!(attr&DB_SHA256)){
+    checked_free(line->sha256);
+  }
+  if(!(attr&DB_SHA512)){
+    checked_free(line->sha512);
+  }
+#ifdef WITH_ACL
+  if(!(attr&DB_ACL)){
+    if (line->acl)
+    {
+      free(line->acl->acl_a);
+      free(line->acl->acl_d);
+    }
+    checked_free(line->acl);
+  }
+#endif
+#ifdef WITH_XATTR
+  if(!(attr&DB_XATTRS)){
+    if (line->xattrs)
+      free(line->xattrs->ents);
+    checked_free(line->xattrs);
+  }
+#endif
+#ifdef WITH_SELINUX
+  if(!(attr&DB_SELINUX)){
+    checked_free(line->cntx);
   }
 #endif
 }
@@ -950,18 +1144,19 @@ void strip_dbline(db_line* line,int attr)
  * status = what to do with this node
  * attr attributes to add 
  */
-void add_file_to_tree(seltree* tree,db_line* file,int db,int status,int attr)
+void add_file_to_tree(seltree* tree,db_line* file,int db,int status,
+                      DB_ATTR_TYPE attr)
 {
   seltree* node=NULL;
-  int localignorelist=0;
-  int ignorelist=0;
+  DB_ATTR_TYPE localignorelist=0;
+  DB_ATTR_TYPE ignorelist=0;
 
   node=get_seltree_node(tree,file->filename);
 
   if(!node){
     node=new_seltree_node(tree,file->filename,0,NULL);
   }
-
+  
   if(file==NULL){
     error(0, "add_file_to_tree was called with NULL db_line\n");
   }
@@ -992,14 +1187,15 @@ void add_file_to_tree(seltree* tree,db_line* file,int db,int status,int attr)
 
   ignorelist=get_groupval("ignore_list");
 
-  if (ignorelist==-1) {
+  if (ignorelist==DB_ATTR_UNDEF) {
     ignorelist=0;
   }
 
   if((node->checked&DB_OLD)&&(node->checked&DB_NEW)){
     localignorelist=(node->new_data->attr^node->old_data->attr);
     if (localignorelist!=0) {
-      error(5,"File %s in databases has different attributes, %i,%i\n",node->old_data->filename,node->old_data->attr,node->new_data->attr);
+      error(2,"File %s in databases has different attributes, %llx,%llx\n",
+            node->old_data->filename,node->old_data->attr,node->new_data->attr);
     }
     
     localignorelist|=ignorelist;
@@ -1016,11 +1212,63 @@ void add_file_to_tree(seltree* tree,db_line* file,int db,int status,int attr)
       
       node->old_data=NULL;
       node->new_data=NULL;      
+      node->checked-=(DB_OLD + DB_NEW); /* mark old and new files as released */          
     }
   }
+
+  /* Do verification if file was moved only if we are asked for it.
+   * old and new data are NULL only if file present in both DBs
+   * and has not been changed.
+   */
+  if( (node->old_data!=NULL || node->new_data!=NULL) &&
+    (file->attr & DB_CHECKINODE)) {
+    /* Check if file was moved (same inode, different name in the other DB)*/
+    db_line *oldData;
+    db_line *newData;
+    seltree* moved_node;
+
+    moved_node=get_seltree_inode(tree,file,db==DB_OLD?DB_NEW:DB_OLD);
+    if(!(moved_node == NULL || moved_node == node)) {
+        /* There's mo match for inode or it matches the node with the same name.
+         * In first case we don't have a match to compare with.
+         * In the second - we already compared those files. */
+      if(db == DB_NEW) {
+        newData = node->new_data;
+        oldData = moved_node->old_data;
+      } else {
+        newData = moved_node->new_data;
+        oldData = node->old_data;
+      }
+
+      localignorelist=(oldData->attr^newData->attr)&(~(DB_NEWFILE|DB_RMFILE));
+
+      if (localignorelist!=0) {
+        error(5,"File \"%s\" \"%s\" in databases has different attributes (here3), %llx,%llx\n",
+  	    newData->filename,oldData->filename,oldData->attr,newData->attr);
+      }
+    
+      localignorelist|=ignorelist|DB_CTIME;
+
+      /* Free the data if same else leave as is for report_tree */
+      if(compare_dbline(oldData, newData, localignorelist)==RETOK){
+        node->checked |= db==DB_NEW ? NODE_MOVED_IN : NODE_MOVED_OUT;
+        moved_node->checked |= db==DB_NEW ? NODE_MOVED_OUT : NODE_MOVED_IN;
+      }
+    }
+  }
+  if( (db == DB_NEW) &&
+      (node->new_data!=NULL) &&
+      (file->attr & DB_NEWFILE) ){
+	 node->checked|=NODE_ALLOW_NEW;
+  }
+  if( (db == DB_OLD) &&
+      (node->old_data!=NULL) &&
+      (file->attr & DB_RMFILE) ){
+	  node->checked|=NODE_ALLOW_RM;
+  }
 }
 
-int check_rxtree(char* filename,seltree* tree,int* attr)
+int check_rxtree(char* filename,seltree* tree,DB_ATTR_TYPE* attr)
 {
   int retval=0;
   char * tmp=NULL;
@@ -1053,7 +1301,7 @@ int check_rxtree(char* filename,seltree* tree,int* attr)
   return retval;
 }
 
-db_line* get_file_attrs(char* filename,int attr)
+db_line* get_file_attrs(char* filename,DB_ATTR_TYPE attr)
 {
   struct AIDE_STAT_TYPE fs;
   int sres=0;
@@ -1070,6 +1318,8 @@ db_line* get_file_attrs(char* filename,int attr)
     }
     return NULL;
   } 
+  if(!(attr&DB_RDEV))
+    fs.st_rdev=0;
   /*
     Get current time for future time notification.
    */
@@ -1133,9 +1383,11 @@ db_line* get_file_attrs(char* filename,int attr)
   /*
     ACL stuff
   */
-#ifdef WITH_ACL
   acl2line(line);
-#endif
+
+  xattrs2line(line);
+
+  selinux2line(line);
 
   if (attr&DB_HASHES && S_ISREG(fs.st_mode)) {
     calc_md(&fs,line);
@@ -1150,69 +1402,6 @@ db_line* get_file_attrs(char* filename,int attr)
   return line;
 }
 
-#if 0
-void stat_dir(char* dir)
-{
-  DIR* dirh=NULL;
-  struct AIDE_DIRENT_TYPE* entp=NULL;
-  struct AIDE_DIRENT_TYPE** resp=NULL;
-  int rdres=0;
-  int add=0;
-  
-  if(!(dirh=opendir(dir))){
-    error(5,"populate_tree():%s: %s\n", strerror(errno),tree->path);
-    return;
-  }
-  
-#ifdef HAVE_READDIR_R
-  resp=(struct AIDE_DIRENT_TYPE**)
-    malloc(sizeof(struct AIDE_DIRENT_TYPE)+_POSIX_PATH_MAX);
-  entp=(struct AIDE_DIRENT_TYPE*)
-    malloc(sizeof(struct AIDE_DIRENT_TYPE)+_POSIX_PATH_MAX);
-  
-  for(rdres=AIDE_READDIR_R_FUNC(dirh,entp,resp);
-      (rdres==0&&(*resp)!=NULL);
-      rdres=AIDE_READDIR_R_FUNC(dirh,entp,resp)){
-#else
-#ifdef HAVE_READDIR
-    for(entp=AIDE_READDIR_FUNC(dirh);
-	(entp!=NULL&&td!=telldir(dirh));
-	entp=AIDE_READDIR_FUNC(dirh)){
-      td=telldir(dirh);
-#else
-#error AIDE needs readdir or readdir_r
-#endif
-#endif
-      if(strncmp(entp->d_name,".",1)==0){
-	if(strncmp(entp->d_name,".",strlen(entp->d_name))==0)
-	  continue;
-	if(strncmp(entp->d_name,"..",strlen(entp->d_name))==0)
-	  continue;
-      }
-      /* Construct fully qualified pathname for the file in question */
-      fullname=(char*)
-	malloc(sizeof(char)*(strlen(entp->d_name)+strlen(tree->path)+2));
-      strncpy(fullname,tree->path,strlen(tree->path));
-      if(strncmp(tree->path,"/",strlen(tree->path))!=0){
-	strncpy(fullname+strlen(tree->path),"/",1);
-	    e=1;
-      }else {
-	e=0;
-      }
-      strncpy(fullname+strlen(tree->path)+e,entp->d_name,strlen(entp->d_name));
-      fullname[(strlen(tree->path)+e+strlen(entp->d_name))]='\0';
-      error(230,_("Checking %s for match\n"),fullname);
-      add=check_rxtree("/",tree,&attr);
-#ifdef HAVE_READDIR_R
-    }
-#else
-#ifdef HAVE_READDIR
-  }
-#endif
-#endif
-}
-#endif
-
 void populate_tree(seltree* tree)
 {
   /* FIXME this function could really use threads */
@@ -1221,8 +1410,8 @@ void populate_tree(seltree* tree)
   db_line* old=NULL;
   db_line* new=NULL;
   int initdbwarningprinted=0;
-  int ignorelist=0;
-  int attr=0;
+  DB_ATTR_TYPE ignorelist=0;
+  DB_ATTR_TYPE attr=0;
   seltree* node=NULL;
   
   /* With this we avoid unnecessary checking of removed files. */
@@ -1234,7 +1423,7 @@ void populate_tree(seltree* tree)
   
   ignorelist=get_groupval("ignore_list");
 
-  if (ignorelist==-1) {
+  if (ignorelist==DB_ATTR_UNDEF) {
     ignorelist=0;
   }
   
@@ -1252,17 +1441,16 @@ void populate_tree(seltree* tree)
 	  node=new_seltree_node(tree,old->filename,0,NULL);
 	}
 	if((add=check_rxtree(old->filename,tree,&attr))>0){
-	  if(add==1){
-	    add_file_to_tree(tree,old,DB_OLD,NODE_ADD_CHILDREN,attr);
-	  }else {
-	    if(add==2){
-	      add_file_to_tree(tree,old,DB_OLD,0,attr);
-	    }
-	  }
+	  add_file_to_tree(tree,old,DB_OLD,0,attr);
 	  i++;
-	}else if(!initdbwarningprinted){
-	  error(5,_("WARNING: Old db contains a file that shouldn\'t be there, run --init or --update\n"));
-	  initdbwarningprinted=1;
+	}else{
+          free_db_line(old);
+          free(old);
+          old=NULL;
+          if(!initdbwarningprinted){
+	    error(3,_("WARNING: Old db contains a file that shouldn\'t be there, run --init or --update\n"));
+	    initdbwarningprinted=1;
+	  }
 	}
 	if(i<100){
 	  old=db_readline(DB_OLD);
@@ -1283,12 +1471,12 @@ void populate_tree(seltree* tree)
 	  node=new_seltree_node(tree,new->filename,0,NULL);
 	}
 	if((add=check_rxtree(new->filename,tree,&attr))>0){
-	  if(add==1){
-	    add_file_to_tree(tree,new,DB_NEW,NODE_ADD_CHILDREN,attr);
-	  }else if (add==2) {
-	    add_file_to_tree(tree,new,DB_NEW,0,attr);
-	  }
+	  add_file_to_tree(tree,new,DB_NEW,0,attr);
 	  i++;
+	} else {
+          free_db_line(new);
+          free(new);
+          new=NULL;
 	}
 	if(i<100){
 	  new=db_readline(DB_NEW);
@@ -1308,16 +1496,14 @@ void populate_tree(seltree* tree)
 	/* Populate tree only if it is needed later */
 	if(conf->action&DO_COMPARE){
 	  if((add=check_rxtree(new->filename,tree,&attr))>0){
-	    if(add==1){
-	      add_file_to_tree(tree,new,DB_NEW,NODE_ADD_CHILDREN,attr);
-	    }else if (add==2) {
-	      add_file_to_tree(tree,new,DB_NEW,0,attr);
-	    }
+	    add_file_to_tree(tree,new,DB_NEW,0,attr);
 	    i++;
 	  }
 	}
 	if((conf->action&DO_INIT)&&!(conf->action&DO_COMPARE)){
 	  free_db_line(new);
+          free(new);
+          new=NULL;
 	}
 	if(i<100){
 	  new=db_readline(DB_DISK);
@@ -1343,28 +1529,15 @@ void hsymlnk(db_line* line) {
       int sres;
       sres=AIDE_STAT_FUNC(line->filename,&fs);
       if (sres!=0 && sres!=EACCES) {
-	error(5,"Dead symlink detected at %s\n",line->filename);
+	error(4,"Dead symlink detected at %s\n",line->filename);
       }
+      if(!(line->attr&DB_RDEV))
+	fs.st_rdev=0;
     }
     /*
       Is this valid?? 
       No, We should do this elsewhere.
     */
-    /*    if(conf->symlinks_found==0){ 
-      int it=0; 
-      DB_FIELD dbtmp; 
-      DB_FIELD dbtmp2; 
-      dbtmp=conf->db_out_order[1]; 
-      conf->db_out_order[1]=db_linkname; 
-      for(it=2;it<conf->db_out_size;it++){ 
-      	dbtmp2=conf->db_out_order[it]; 
-      	conf->db_out_order[it]=dbtmp;
-      	dbtmp=dbtmp2;
-      } 
-      conf->db_out_order[conf->db_out_size++]=dbtmp; 
-      conf->symlinks_found=1; 
-    } 
-    */
     line->linkname=(char*)malloc(_POSIX_PATH_MAX+1);
     if(line->linkname==NULL){
       error(0,_("malloc failed in add_file_to_list()\n"));
@@ -1385,18 +1558,6 @@ void hsymlnk(db_line* line) {
     
     len=readlink(line->filename,line->linkname,_POSIX_PATH_MAX+1);
     
-    /*
-      lnkbuf=(char*)malloc(len+1);
-      if(lnkbuf==NULL){
-      error(0,_("malloc failed in add_file_to_list()\n"));
-      abort();
-      }
-
-      strncpy(lnkbuf,lnktmp,len);
-      lnkbuf[len]='\0';
-      line->linkname=lnkbuf;
-      free(lnktmp);
-    */
     /*
      * We use realloc :)
      */
@@ -1408,6 +1569,4 @@ void hsymlnk(db_line* line) {
   }
   
 }
-/*
-const char* aide_key_13=KEY_13;
-*/
+// vi: ts=8 sw=2
