@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/list.c,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/src/list.c,v 1.3 2006/10/27 20:44:38 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
 #include <stdlib.h>
 #include "list.h"
 #include "report.h"
@@ -95,34 +96,6 @@ list* list_append(list* listp,void*data)
   return NULL;
 }
 
-/*
- * new_list_item()
- * create a new list item with data
- */
-
-/*
-
-obsolete
-
-list* new_list_item(void* data)
-{
-
-  list* item=NULL;
-
-  item=(list*)malloc(sizeof(list));
-  item->prev=NULL;
-  item->next=NULL;
-  item->head=item;
-  item->tail=item;
-  item->data=data;
-  
-  abort();
-
-  return NULL;
-}
-
-*/
-
 /*
  * delete_list_item()
  * delete a item from list
@@ -190,6 +163,3 @@ list* list_delete_item(list* item){
   return r;
   
 }
-/*
-const char* aide_key_12=KEY_12;
-*/
