@@ -1,7 +1,7 @@
 /* Aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/md.c,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/src/md.c,v 1.6 2006/11/29 12:58:02 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,8 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
 #include <stdlib.h>
-#include "../config.h"
 #include "md.h"
 #include "report.h"
 #include <string.h>
@@ -27,7 +27,6 @@
 #include <mhash.h>
 #endif
 #define HASH_HAVAL_LEN HASH_HAVAL256_LEN
-/*.#define haval haval256.*/
 
 
 /*
@@ -35,8 +34,52 @@
   stored. Only a speed issue.
 */
 
-int hash_mhash2attr(int i) {
-  int r=0;
+DB_ATTR_TYPE hash_gcrypt2attr(int i) {
+  DB_ATTR_TYPE r=0;
+#ifdef WITH_GCRYPT
+  switch (i) {
+  case GCRY_MD_MD5: {
+    r=DB_MD5;
+    break;
+  }
+  case GCRY_MD_SHA1: {
+    r=DB_SHA1;
+    break;
+  }
+  case GCRY_MD_RMD160: {
+    r=DB_RMD160;
+    break;
+  }
+  case GCRY_MD_TIGER: {
+    r=DB_TIGER;
+    break;
+  }
+  case GCRY_MD_HAVAL: {
+    r=DB_HAVAL;
+    break;
+  }
+  case GCRY_MD_SHA256: {
+    r=DB_SHA256;
+    break;
+  }
+  case GCRY_MD_SHA512: {
+    r=DB_SHA512;
+    break;
+  }
+  case GCRY_MD_CRC32: {
+    r=DB_CRC32;
+    break;
+  }
+  default:
+    break;
+  }
+#endif
+  return r;
+}
+
+DB_ATTR_TYPE hash_mhash2attr(int i) {
+  DB_ATTR_TYPE r=0;
+#ifdef WITH_MHASH
   switch (i) {
   case MHASH_CRC32: {
     r=DB_CRC32;
@@ -92,15 +135,26 @@ int hash_mhash2attr(int i) {
     break;
   }
   case MHASH_SHA256: {
+    r=DB_SHA256;
     break;
   }
+  case MHASH_SHA512: {
+    r=DB_SHA512;
+    break;
+  }
+#ifdef HAVE_MHASH_WHIRLPOOL		 
+  case MHASH_WHIRLPOOL: {
+    r=DB_WHIRLPOOL;
+    break;
+  }
+#endif
   case MHASH_ADLER32: {
     break;
   }
   default:
     break;
   }
-
+#endif
   return r;
 }
 
@@ -111,7 +165,7 @@ int hash_mhash2attr(int i) {
 int init_md(struct md_container* md) {
   
   int i;
-  /*.    First we check theparameter..   */
+  /*    First we check the parameter..   */
 #ifdef _PARAMETER_CHECK_
   if (md==NULL) {
     return RETFAIL;  
@@ -125,19 +179,19 @@ int init_md(struct md_container* md) {
 #ifdef WITH_MHASH
   error(255,"Mhash library initialization\n");
   for(i=0;i<=HASH_MHASH_COUNT;i++) {
-    if (((hash_mhash2attr(i))&(HASH_USE_MHASH)&md->todo_attr)!=0) {
-      int h=hash_mhash2attr(i);
-      error(255,"inserting %i\n",h);
+    if (((hash_mhash2attr(i)&HASH_USE_MHASH)&md->todo_attr)!=0) {
+      DB_ATTR_TYPE h=hash_mhash2attr(i);
+      error(255,"inserting %llu\n",h);
       md->mhash_mdh[i]=mhash_init(i);
       if (md->mhash_mdh[i]!=MHASH_FAILED) {
-	md->calc_attr|=h;
+				md->calc_attr|=h;
       } else {
 	/*
 	  Oops.. 
 	  We just don't calculate this.
 	 */
-	
-	md->todo_attr&=~h;
+
+				md->todo_attr&=~h;
       }
 
     } else {
@@ -145,6 +199,25 @@ int init_md(struct md_container* md) {
     }
   }
 #endif 
+#ifdef WITH_GCRYPT
+  error(255,"Gcrypt library initialization\n");
+	if(gcry_md_open(&md->mdh,0,0)!=GPG_ERR_NO_ERROR){
+		error(0,"gcrypt_md_open failed\n");
+		exit(IO_ERROR);
+	}
+  for(i=0;i<=HASH_GCRYPT_COUNT;i++) {
+    if (((hash_gcrypt2attr(i)&HASH_USE_GCRYPT)&md->todo_attr)!=0) {
+      DB_ATTR_TYPE h=hash_gcrypt2attr(i);
+      error(255,"inserting %llu\n",h);
+			if(gcry_md_enable(md->mdh,i)==GPG_ERR_NO_ERROR){
+				md->calc_attr|=h;
+			} else {
+				error(0,"gcry_md_enable %i failed",i);
+				md->todo_attr&=~h;
+			}
+		}
+	}
+#endif
   return RETOK;
 }
 
@@ -173,6 +246,9 @@ int update_md(struct md_container* md,void* data,ssize_t size) {
   }
   
 #endif /* WITH_MHASH */
+#ifdef WITH_GCRYPT
+	gcry_md_write(md->mdh, data, size);
+#endif
   return RETOK;
 }
 
@@ -196,20 +272,23 @@ int close_md(struct md_container* md) {
     }  
   }
 #endif /* WITH_MHASH */
-#ifdef WITH_LIBGCRYPT
-  gcry_md_write(mdh,(byte*)buf,0);
+#ifdef WITH_GCRYPT
   gcry_md_final(md->mdh); 
   /* Let's flush the buffers */
 
 #define get_libgcrypt_hash(a,b,c,d) \
-  if(md->calc_attr&a&HASH_USE_LIBGCRYPT){\
-    memcpy(md->b,gcry_md_read(md->mdh,c),d);\
+  if(md->calc_attr&a&HASH_USE_GCRYPT){\
+		error(255,"Getting hash %i\n",b);\
+    memcpy(md->c,gcry_md_read(md->mdh,b),d);\
   }
 
   get_libgcrypt_hash(DB_MD5,GCRY_MD_MD5,md5,HASH_MD5_LEN);
   get_libgcrypt_hash(DB_SHA1,GCRY_MD_SHA1,sha1,HASH_SHA1_LEN);
   get_libgcrypt_hash(DB_TIGER,GCRY_MD_TIGER,tiger,HASH_TIGER_LEN);
-  get_libgcrypt_hash(DB_RMD160,GCRY_MD_RMD160,rmd160,HASH_RMD160_LEN);  
+  get_libgcrypt_hash(DB_RMD160,GCRY_MD_RMD160,rmd160,HASH_RMD160_LEN);
+  get_libgcrypt_hash(DB_SHA256,GCRY_MD_SHA256,sha256,HASH_SHA256_LEN);
+  get_libgcrypt_hash(DB_SHA512,GCRY_MD_SHA512,sha512,HASH_SHA512_LEN);
+  get_libgcrypt_hash(DB_CRC32,GCRY_MD_CRC32,crc32,HASH_CRC32_LEN);
   
   /*.    There might be more hashes in the library. Add those here..   */
   
@@ -230,6 +309,11 @@ int close_md(struct md_container* md) {
   get_mhash_hash(MHASH_HAVAL,haval);
   get_mhash_hash(MHASH_GOST,gost);
   get_mhash_hash(MHASH_CRC32B,crc32b);
+  get_mhash_hash(MHASH_SHA256,sha256);
+  get_mhash_hash(MHASH_SHA512,sha512);
+#ifdef HAVE_MHASH_WHIRLPOOL
+  get_mhash_hash(MHASH_WHIRLPOOL,whirlpool);
+#endif
   
   /*
     There might be more hashes in the library we want to use.
@@ -275,7 +359,8 @@ void md2line(struct md_container* md,struct db_line* line) {
   copyhash(DB_HAVAL,haval,HASH_HAVAL_LEN);
   copyhash(DB_GOST,gost,HASH_GOST_LEN);
   copyhash(DB_CRC32B,crc32b,HASH_CRC32B_LEN);
+
+  copyhash(DB_SHA256,sha256,HASH_SHA256_LEN);
+  copyhash(DB_SHA512,sha512,HASH_SHA512_LEN);
+  copyhash(DB_WHIRLPOOL,whirlpool,HASH_WHIRLPOOL_LEN);
 }
-/*
-  const char* aide_key_14=KEY_14;
-*/
