@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
- * $Header: /cvsroot/aide/aide/src/util.c,v 1.1.1.1 2003/01/16 10:37:34 rammer Exp $
+ * $Header: /cvsroot/aide/aide/src/util.c,v 1.10 2006/11/24 20:47:01 rvdb Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,12 +18,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "aide.h"
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <signal.h>
 #include <ctype.h>
+#include <syslog.h>
 /*for locale support*/
 #include "locale-aide.h"
 /*for locale support*/
@@ -36,16 +38,15 @@
 #include "report.h"
 #include "db_config.h"
 #include "util.h"
-#include "aide.h"
 
 #define URL_UNSAFE " <>\"#%{}|\\^~[]`@:\033'"
 #define ISPRINT(c) (isascii(c) && isprint(c))
 
 static const char* url_name[] = { 
-  "file", "stdin", "stdout", "stderr", "fd", "sql", "syslog", "database" };
+  "file", "stdin", "stdout", "stderr", "fd", "sql", "syslog", "database", "https", "http", "ftp" };
 
 static const int url_value[] = {
-  url_file, url_stdin, url_stdout,url_stderr,url_fd, url_sql, url_syslog, url_database };
+  url_file, url_stdin, url_stdout,url_stderr,url_fd, url_sql, url_syslog, url_database, url_https, url_http, url_ftp };
 
 const int url_ntypes=sizeof(url_value)/sizeof(URL_TYPE);
 
@@ -90,7 +91,6 @@ url_t* parse_url(char* val)
     }
   }
   
-  
   switch (u->type) {
   case url_file : {
     if(r[0]=='/'&&(r+1)[0]=='/'&&(r+2)[0]=='/'){
@@ -127,6 +127,12 @@ url_t* parse_url(char* val)
     
     break;
   }
+  case url_https :
+  case url_http :
+  case url_ftp : {
+    u->value=strdup(val);
+    break;
+  }
   case url_unknown : {
     error(0,"Unknown URL-type:%s\n",val_copy);
     break;
@@ -322,7 +328,7 @@ void sig_handler(int signal)
   case SIGSEGV :{
     error(200,"Caught SIGBUS/SIGSEGV\n");
     if(conf->catch_mmap==1){
-      error(5,"Caught SIGBUS/SEGV while mmapping. File was truncated while aide was running?\n");
+      error(4,"Caught SIGBUS/SEGV while mmapping. File was truncated while aide was running?\n");
       conf->catch_mmap=0;
     } else {
       error(0,"Caught SIGBUS/SEGV. Exiting\n");
@@ -332,20 +338,20 @@ void sig_handler(int signal)
     break;
   }
   case SIGHUP : {
-    error(5,"Caught SIGHUP\n");
+    error(4,"Caught SIGHUP\n");
     break;
   }
   case SIGTERM : {
-    error(5,"Caught SIGTERM\nUse SIGKILL to terminate\n");
+    error(4,"Caught SIGTERM\nUse SIGKILL to terminate\n");
     break;
   }
   case SIGUSR1 : {
-    error(5,"Setting output to debug level according to signal\n");
+    error(4,"Setting output to debug level according to signal\n");
     conf->verbose_level=220;
     break;
   }
   case SIGUSR2 : {
-    error(5,"Setting output to normal level according to signal\n");
+    error(4,"Setting output to normal level according to signal\n");
     conf->verbose_level=5;
     break;
   }
@@ -362,6 +368,7 @@ void sig_handler(int signal)
    needle has to be NULL terminated. NULL in needle is not used in compare.
    NULLs in haystack are ignored.
 */
+#ifndef HAVE_STRNSTR
 char* strnstr(char* haystack,char* needle,int n)
 {
   char* h=haystack;
@@ -392,6 +399,100 @@ char* strnstr(char* haystack,char* needle,int n)
   /* If we get this far no match was found so we return NULL */
   return NULL;
 }
+#endif
+
+#ifndef HAVE_STRNLEN
+size_t strnlen(const char *s, size_t maxlen)
+{
+	size_t l;
+	l=strlen(s);
+	if(l>maxlen)
+		return maxlen;
+	return l;
+}
+#endif
+
+/* Lookup syslog facilities by name */
+int syslog_facility_lookup(char *s)
+{
+	if(!s || strlen(s)<1)
+		return(AIDE_SYSLOG_FACILITY);
+#ifdef LOG_KERN
+	if(strcasecmp(s,"LOG_KERN")==0)
+		return(LOG_KERN);
+#endif
+#ifdef LOG_USER
+	if(strcasecmp(s,"LOG_USER")==0)
+		return(LOG_USER);
+#endif
+#ifdef LOG_MAIL
+	if(strcasecmp(s,"LOG_MAIL")==0)
+		return(LOG_MAIL);
+#endif
+#ifdef LOG_DAEMON
+	if(strcasecmp(s,"LOG_DAEMON")==0)
+		return(LOG_DAEMON);
+#endif
+#ifdef LOG_AUTH
+	if(strcasecmp(s,"LOG_AUTH")==0)
+		return(LOG_AUTH);
+#endif
+#ifdef LOG_SYSLOG
+	if(strcasecmp(s,"LOG_SYSLOG")==0)
+		return(LOG_SYSLOG);
+#endif
+#ifdef LOG_LPR
+	if(strcasecmp(s,"LOG_LPR")==0)
+		return(LOG_LPR);
+#endif
+#ifdef LOG_NEWS
+	if(strcasecmp(s,"LOG_NEWS")==0)
+		return(LOG_NEWS);
+#endif
+#ifdef LOG_UUCP
+	if(strcasecmp(s,"LOG_UUCP")==0)
+		return(LOG_UUCP);
+#endif
+#ifdef LOG_CRON
+	if(strcasecmp(s,"LOG_CRON")==0)
+		return(LOG_CRON);
+#endif
+#ifdef LOG_LOCAL0
+	if(strcasecmp(s,"LOG_LOCAL0")==0)
+		return(LOG_LOCAL0);
+#endif
+#ifdef LOG_LOCAL1
+	if(strcasecmp(s,"LOG_LOCAL1")==0)
+		return(LOG_LOCAL1);
+#endif
+#ifdef LOG_LOCAL2
+	if(strcasecmp(s,"LOG_LOCAL2")==0)
+		return(LOG_LOCAL2);
+#endif
+#ifdef LOG_LOCAL3
+	if(strcasecmp(s,"LOG_LOCAL3")==0)
+		return(LOG_LOCAL3);
+#endif
+#ifdef LOG_LOCAL4
+	if(strcasecmp(s,"LOG_LOCAL4")==0)
+		return(LOG_LOCAL4);
+#endif
+#ifdef LOG_LOCAL5
+	if(strcasecmp(s,"LOG_LOCAL5")==0)
+		return(LOG_LOCAL5);
+#endif
+#ifdef LOG_LOCAL6
+	if(strcasecmp(s,"LOG_LOCAL6")==0)
+		return(LOG_LOCAL6);
+#endif
+#ifdef LOG_LOCAL7
+	if(strcasecmp(s,"LOG_LOCAL7")==0)
+		return(LOG_LOCAL7);
+#endif
+
+	error(0,"Syslog facility \"%s\" is unknown, using default\n",s);
+	return(AIDE_SYSLOG_FACILITY);
+}
 
 /* We need these dummy stubs to fool the linker into believing that
    we do not need them at link time */
